# This file is generated by objective.metadata
#
# Last update: Thu Aug  8 16:37:57 2019
#
# flake8: noqa

import sys

import objc

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
misc.update(
    {
        "CATransform3D": objc.createStructType(
            "CATransform3D",
            sel32or64(
                b"{CATransform3D=ffffffffffffffff}", b"{CATransform3D=dddddddddddddddd}"
            ),
            [
                "m11",
                "m12",
                "m13",
                "m14",
                "m21",
                "m22",
                "m23",
                "m24",
                "m31",
                "m32",
                "m33",
                "m34",
                "m41",
                "m42",
                "m43",
                "m44",
            ],
        )
    }
)
constants = """$CIDetectorAccuracy$CIDetectorAccuracyHigh$CIDetectorAccuracyLow$CIDetectorAspectRatio$CIDetectorEyeBlink$CIDetectorFocalLength$CIDetectorImageOrientation$CIDetectorMaxFeatureCount$CIDetectorMinFeatureSize$CIDetectorNumberOfAngles$CIDetectorReturnSubFeatures$CIDetectorSmile$CIDetectorTracking$CIDetectorTypeFace$CIDetectorTypeQRCode$CIDetectorTypeRectangle$CIDetectorTypeText$CIFeatureTypeFace$CIFeatureTypeQRCode$CIFeatureTypeRectangle$CIFeatureTypeText$kCAAlignmentCenter$kCAAlignmentJustified$kCAAlignmentLeft$kCAAlignmentNatural$kCAAlignmentRight$kCAAnimationCubic$kCAAnimationCubicPaced$kCAAnimationDiscrete$kCAAnimationLinear$kCAAnimationPaced$kCAAnimationRotateAuto$kCAAnimationRotateAutoReverse$kCAContentsFormatGray8Uint$kCAContentsFormatRGBA16Float$kCAContentsFormatRGBA8Uint$kCACornerCurveCircular$kCACornerCurveContinuous$kCAEmitterBehaviorAlignToMotion$kCAEmitterBehaviorAttractor$kCAEmitterBehaviorColorOverLife$kCAEmitterBehaviorDrag$kCAEmitterBehaviorLight$kCAEmitterBehaviorSimpleAttractor$kCAEmitterBehaviorValueOverLife$kCAEmitterBehaviorWave$kCAEmitterLayerAdditive$kCAEmitterLayerBackToFront$kCAEmitterLayerCircle$kCAEmitterLayerCuboid$kCAEmitterLayerLine$kCAEmitterLayerOldestFirst$kCAEmitterLayerOldestLast$kCAEmitterLayerOutline$kCAEmitterLayerPoint$kCAEmitterLayerPoints$kCAEmitterLayerRectangle$kCAEmitterLayerSphere$kCAEmitterLayerSurface$kCAEmitterLayerUnordered$kCAEmitterLayerVolume$kCAFillModeBackwards$kCAFillModeBoth$kCAFillModeForwards$kCAFillModeFrozen$kCAFillModeRemoved$kCAFillRuleEvenOdd$kCAFillRuleNonZero$kCAFilterLinear$kCAFilterNearest$kCAFilterTrilinear$kCAGradientLayerAxial$kCAGradientLayerConic$kCAGradientLayerRadial$kCAGravityBottom$kCAGravityBottomLeft$kCAGravityBottomRight$kCAGravityCenter$kCAGravityLeft$kCAGravityResize$kCAGravityResizeAspect$kCAGravityResizeAspectFill$kCAGravityRight$kCAGravityTop$kCAGravityTopLeft$kCAGravityTopRight$kCALineCapButt$kCALineCapRound$kCALineCapSquare$kCALineJoinBevel$kCALineJoinMiter$kCALineJoinRound$kCAMediaTimingFunctionDefault$kCAMediaTimingFunctionEaseIn$kCAMediaTimingFunctionEaseInEaseOut$kCAMediaTimingFunctionEaseOut$kCAMediaTimingFunctionLinear$kCAOnOrderIn$kCAOnOrderOut$kCARendererColorSpace$kCARendererMetalCommandQueue$kCAScrollBoth$kCAScrollHorizontally$kCAScrollNone$kCAScrollVertically$kCATransactionAnimationDuration$kCATransactionAnimationTimingFunction$kCATransactionCompletionBlock$kCATransactionDisableActions$kCATransition$kCATransitionFade$kCATransitionFromBottom$kCATransitionFromLeft$kCATransitionFromRight$kCATransitionFromTop$kCATransitionMoveIn$kCATransitionPush$kCATransitionReveal$kCATruncationEnd$kCATruncationMiddle$kCATruncationNone$kCATruncationStart$kCAValueFunctionRotateX$kCAValueFunctionRotateY$kCAValueFunctionRotateZ$kCAValueFunctionScale$kCAValueFunctionScaleX$kCAValueFunctionScaleY$kCAValueFunctionScaleZ$kCAValueFunctionTranslate$kCAValueFunctionTranslateX$kCAValueFunctionTranslateY$kCAValueFunctionTranslateZ$kCIActiveKeys$kCIApplyOptionColorSpace$kCIApplyOptionDefinition$kCIApplyOptionExtent$kCIApplyOptionUserInfo$kCIAttributeClass$kCIAttributeDefault$kCIAttributeDescription$kCIAttributeDisplayName$kCIAttributeFilterAvailable_Mac$kCIAttributeFilterAvailable_iOS$kCIAttributeFilterCategories$kCIAttributeFilterDisplayName$kCIAttributeFilterName$kCIAttributeIdentity$kCIAttributeMax$kCIAttributeMin$kCIAttributeName$kCIAttributeReferenceDocumentation$kCIAttributeSliderMax$kCIAttributeSliderMin$kCIAttributeType$kCIAttributeTypeAngle$kCIAttributeTypeBoolean$kCIAttributeTypeColor$kCIAttributeTypeCount$kCIAttributeTypeDistance$kCIAttributeTypeGradient$kCIAttributeTypeImage$kCIAttributeTypeInteger$kCIAttributeTypeOffset$kCIAttributeTypeOpaqueColor$kCIAttributeTypePosition$kCIAttributeTypePosition3$kCIAttributeTypeRectangle$kCIAttributeTypeScalar$kCIAttributeTypeTime$kCIAttributeTypeTransform$kCICategoryBlur$kCICategoryBuiltIn$kCICategoryColorAdjustment$kCICategoryColorEffect$kCICategoryCompositeOperation$kCICategoryDistortionEffect$kCICategoryFilterGenerator$kCICategoryGenerator$kCICategoryGeometryAdjustment$kCICategoryGradient$kCICategoryHalftoneEffect$kCICategoryHighDynamicRange$kCICategoryInterlaced$kCICategoryNonSquarePixels$kCICategoryReduction$kCICategorySharpen$kCICategoryStillImage$kCICategoryStylize$kCICategoryTileEffect$kCICategoryTransition$kCICategoryVideo$kCIContextAllowLowPower$kCIContextCacheIntermediates$kCIContextHighQualityDownsample$kCIContextOutputColorSpace$kCIContextOutputPremultiplied$kCIContextPriorityRequestLow$kCIContextUseSoftwareRenderer$kCIContextWorkingColorSpace$kCIContextWorkingFormat$kCIFilterGeneratorExportedKey$kCIFilterGeneratorExportedKeyName$kCIFilterGeneratorExportedKeyTargetObject$kCIFormatA16@i$kCIFormatA8@i$kCIFormatABGR8@i$kCIFormatARGB8@i$kCIFormatAf@i$kCIFormatAh@i$kCIFormatBGRA8@i$kCIFormatL16@i$kCIFormatL8@i$kCIFormatLA16@i$kCIFormatLA8@i$kCIFormatLAf@i$kCIFormatLAh@i$kCIFormatLf@i$kCIFormatLh@i$kCIFormatR16@i$kCIFormatR8@i$kCIFormatRG16@i$kCIFormatRG8@i$kCIFormatRGBA16@i$kCIFormatRGBA8@i$kCIFormatRGBAf@i$kCIFormatRGBAh@i$kCIFormatRGf@i$kCIFormatRGh@i$kCIFormatRf@i$kCIFormatRh@i$kCIImageApplyOrientationProperty$kCIImageAutoAdjustCrop$kCIImageAutoAdjustEnhance$kCIImageAutoAdjustFeatures$kCIImageAutoAdjustLevel$kCIImageAutoAdjustRedEye$kCIImageAuxiliaryDepth$kCIImageAuxiliaryDisparity$kCIImageAuxiliaryPortraitEffectsMatte$kCIImageAuxiliarySemanticSegmentationHairMatte$kCIImageAuxiliarySemanticSegmentationSkinMatte$kCIImageAuxiliarySemanticSegmentationTeethMatte$kCIImageColorSpace$kCIImageNearestSampling$kCIImageProperties$kCIImageProviderTileSize$kCIImageProviderUserInfo$kCIImageRepresentationAVDepthData$kCIImageRepresentationAVPortraitEffectsMatte$kCIImageRepresentationAVSemanticSegmentationMattes$kCIImageRepresentationDepthImage$kCIImageRepresentationDisparityImage$kCIImageRepresentationPortraitEffectsMatteImage$kCIImageRepresentationSemanticSegmentationHairMatteImage$kCIImageRepresentationSemanticSegmentationSkinMatteImage$kCIImageRepresentationSemanticSegmentationTeethMatteImage$kCIImageTextureFormat$kCIImageTextureTarget$kCIInputAllowDraftModeKey$kCIInputAmountKey$kCIInputAngleKey$kCIInputAspectRatioKey$kCIInputBackgroundImageKey$kCIInputBaselineExposureKey$kCIInputBiasKey$kCIInputBoostKey$kCIInputBoostShadowAmountKey$kCIInputBrightnessKey$kCIInputCenterKey$kCIInputColorKey$kCIInputColorNoiseReductionAmountKey$kCIInputContrastKey$kCIInputDecoderVersionKey$kCIInputDepthImageKey$kCIInputDisableGamutMapKey$kCIInputDisparityImageKey$kCIInputEVKey$kCIInputEnableChromaticNoiseTrackingKey$kCIInputEnableEDRModeKey$kCIInputEnableSharpeningKey$kCIInputEnableVendorLensCorrectionKey$kCIInputExtentKey$kCIInputGradientImageKey$kCIInputIgnoreImageOrientationKey$kCIInputImageKey$kCIInputImageOrientationKey$kCIInputIntensityKey$kCIInputLinearSpaceFilter$kCIInputLuminanceNoiseReductionAmountKey$kCIInputMaskImageKey$kCIInputMatteImageKey$kCIInputMoireAmountKey$kCIInputNeutralChromaticityXKey$kCIInputNeutralChromaticityYKey$kCIInputNeutralLocationKey$kCIInputNeutralTemperatureKey$kCIInputNeutralTintKey$kCIInputNoiseReductionAmountKey$kCIInputNoiseReductionContrastAmountKey$kCIInputNoiseReductionDetailAmountKey$kCIInputNoiseReductionSharpnessAmountKey$kCIInputRadiusKey$kCIInputRefractionKey$kCIInputSaturationKey$kCIInputScaleFactorKey$kCIInputScaleKey$kCIInputShadingImageKey$kCIInputSharpnessKey$kCIInputTargetImageKey$kCIInputTimeKey$kCIInputTransformKey$kCIInputVersionKey$kCIInputWeightsKey$kCIInputWidthKey$kCIOutputImageKey$kCIOutputNativeSizeKey$kCISamplerAffineMatrix$kCISamplerColorSpace$kCISamplerFilterLinear$kCISamplerFilterMode$kCISamplerFilterNearest$kCISamplerWrapBlack$kCISamplerWrapClamp$kCISamplerWrapMode$kCISupportedDecoderVersionsKey$kCIUIParameterSet$kCIUISetAdvanced$kCIUISetBasic$kCIUISetDevelopment$kCIUISetIntermediate$"""
constants = constants + "$CATransform3DIdentity@%s$" % (
    sel32or64("{CATransform3D=ffffffffffffffff}", "{CATransform3D=dddddddddddddddd}"),
)
enums = """$CA_TEST@0$CA_WARN_DEPRECATED@1$CIDataMatrixCodeECCVersion000@0$CIDataMatrixCodeECCVersion050@50$CIDataMatrixCodeECCVersion080@80$CIDataMatrixCodeECCVersion100@100$CIDataMatrixCodeECCVersion140@140$CIDataMatrixCodeECCVersion200@200$CIQRCodeErrorCorrectionLevelH@72$CIQRCodeErrorCorrectionLevelL@76$CIQRCodeErrorCorrectionLevelM@77$CIQRCodeErrorCorrectionLevelQ@81$CIRenderDestinationAlphaNone@0$CIRenderDestinationAlphaPremultiplied@1$CIRenderDestinationAlphaUnpremultiplied@2$kCAConstraintHeight@7$kCAConstraintMaxX@2$kCAConstraintMaxY@6$kCAConstraintMidX@1$kCAConstraintMidY@5$kCAConstraintMinX@0$kCAConstraintMinY@4$kCAConstraintWidth@3$kCALayerBottomEdge@4$kCALayerHeightSizable@16$kCALayerLeftEdge@1$kCALayerMaxXMargin@4$kCALayerMaxXMaxYCorner@8$kCALayerMaxXMinYCorner@2$kCALayerMaxYMargin@32$kCALayerMinXMargin@1$kCALayerMinXMaxYCorner@4$kCALayerMinXMinYCorner@1$kCALayerMinYMargin@8$kCALayerNotSizable@0$kCALayerRightEdge@2$kCALayerTopEdge@8$kCALayerWidthSizable@2$"""
misc.update({})
functions = {
    "CATransform3DIsAffine": (
        sel32or64(
            b"B{CATransform3D=ffffffffffffffff}", b"B{CATransform3D=dddddddddddddddd}"
        ),
    ),
    "CATransform3DInvert": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}{CATransform3D=ffffffffffffffff}",
            b"{CATransform3D=dddddddddddddddd}{CATransform3D=dddddddddddddddd}",
        ),
    ),
    "CATransform3DIsIdentity": (
        sel32or64(
            b"B{CATransform3D=ffffffffffffffff}", b"B{CATransform3D=dddddddddddddddd}"
        ),
    ),
    "CATransform3DMakeScale": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}fff",
            b"{CATransform3D=dddddddddddddddd}ddd",
        ),
    ),
    "CATransform3DTranslate": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}{CATransform3D=ffffffffffffffff}fff",
            b"{CATransform3D=dddddddddddddddd}{CATransform3D=dddddddddddddddd}ddd",
        ),
    ),
    "CATransform3DEqualToTransform": (
        sel32or64(
            b"B{CATransform3D=ffffffffffffffff}{CATransform3D=ffffffffffffffff}",
            b"B{CATransform3D=dddddddddddddddd}{CATransform3D=dddddddddddddddd}",
        ),
    ),
    "CATransform3DRotate": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}{CATransform3D=ffffffffffffffff}ffff",
            b"{CATransform3D=dddddddddddddddd}{CATransform3D=dddddddddddddddd}dddd",
        ),
    ),
    "CACurrentMediaTime": (b"d",),
    "CATransform3DMakeRotation": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}ffff",
            b"{CATransform3D=dddddddddddddddd}dddd",
        ),
    ),
    "CATransform3DConcat": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}{CATransform3D=ffffffffffffffff}{CATransform3D=ffffffffffffffff}",
            b"{CATransform3D=dddddddddddddddd}{CATransform3D=dddddddddddddddd}{CATransform3D=dddddddddddddddd}",
        ),
    ),
    "CATransform3DScale": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}{CATransform3D=ffffffffffffffff}fff",
            b"{CATransform3D=dddddddddddddddd}{CATransform3D=dddddddddddddddd}ddd",
        ),
    ),
    "CATransform3DMakeTranslation": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}fff",
            b"{CATransform3D=dddddddddddddddd}ddd",
        ),
    ),
    "CATransform3DGetAffineTransform": (
        sel32or64(
            b"{CGAffineTransform=ffffff}{CATransform3D=ffffffffffffffff}",
            b"{CGAffineTransform=dddddd}{CATransform3D=dddddddddddddddd}",
        ),
    ),
    "CATransform3DMakeAffineTransform": (
        sel32or64(
            b"{CATransform3D=ffffffffffffffff}{CGAffineTransform=ffffff}",
            b"{CATransform3D=dddddddddddddddd}{CGAffineTransform=dddddd}",
        ),
    ),
}
aliases = {"CA_TESTABLE_CLASS": "CA_HIDDEN", "CA_TESTABLE": "CA_HIDDEN"}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b"CAAnimation", b"isRemovedOnCompletion", {"retval": {"type": b"Z"}})
    r(b"CAAnimation", b"setEnabled:", {"arguments": {2: {"type": "Z"}}})
    r(b"CAAnimation", b"setRemovedOnCompletion:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CAAnimation", b"shouldArchiveValueForKey:", {"retval": {"type": b"Z"}})
    r(
        b"CADisplayLink",
        b"displayLinkWithTarget:selector:",
        {"arguments": {3: {"sel_of_type": b"v@:@"}}},
    )
    r(b"CADisplayLink", b"isPaused", {"retval": {"type": b"Z"}})
    r(b"CADisplayLink", b"setPaused:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CAEmitterBehavior", b"isEnabled", {"retval": {"type": b"Z"}})
    r(b"CAEmitterBehavior", b"setEnabled:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CAEmitterCell", b"isEnabled", {"retval": {"type": b"Z"}})
    r(b"CAEmitterCell", b"setEnabled:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CAEmitterCell", b"shouldArchiveValueForKey:", {"retval": {"type": b"Z"}})
    r(b"CAEmitterLayer", b"preservesDepth", {"retval": {"type": b"Z"}})
    r(b"CAEmitterLayer", b"setPreservesDepth:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"allowsEdgeAntialiasing", {"retval": {"type": "Z"}})
    r(b"CALayer", b"allowsGroupOpacity", {"retval": {"type": "Z"}})
    r(b"CALayer", b"containsPoint:", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"contentsAreFlipped", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"drawsAsynchronously", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"isDoubleSided", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"isGeometryFlipped", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"isHidden", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"isOpaque", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"masksToBounds", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"needsDisplay", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"needsDisplayForKey:", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"needsDisplayOnBoundsChange", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"needsLayout", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"setAllowsEdgeAntialiasing:", {"arguments": {2: {"type": "Z"}}})
    r(b"CALayer", b"setAllowsGroupOpacity:", {"arguments": {2: {"type": "Z"}}})
    r(b"CALayer", b"setDoubleSided:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"setDrawsAsynchronously:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"setGeometryFlipped:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"setHidden:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"setMasksToBounds:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"setNeedsDisplayOnBoundsChange:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"setOpaque:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CALayer", b"setShouldRasterize:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"CALayer",
        b"setWantsExtendedDynamicRangeContent:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(b"CALayer", b"shouldArchiveValueForKey:", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"shouldRasterize", {"retval": {"type": b"Z"}})
    r(b"CALayer", b"wantsExtendedDynamicRangeContent", {"retval": {"type": "Z"}})
    r(b"CAMetalLayer", b"allowsNextDrawableTimeout", {"retval": {"type": "Z"}})
    r(b"CAMetalLayer", b"displaySyncEnabled", {"retval": {"type": "Z"}})
    r(b"CAMetalLayer", b"framebufferOnly", {"retval": {"type": "Z"}})
    r(b"CAMetalLayer", b"presentsWithTransaction", {"retval": {"type": "Z"}})
    r(
        b"CAMetalLayer",
        b"setAllowsNextDrawableTimeout:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(b"CAMetalLayer", b"setDisplaySyncEnabled:", {"arguments": {2: {"type": "Z"}}})
    r(b"CAMetalLayer", b"setFramebufferOnly:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"CAMetalLayer",
        b"setPresentsWithTransaction:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(
        b"CAMetalLayer",
        b"setWantsExtendedDynamicRangeContent:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(b"CAMetalLayer", b"wantsExtendedDynamicRangeContent", {"retval": {"type": "Z"}})
    r(
        b"CAOpenGLLayer",
        b"canDrawInCGLContext:pixelFormat:forLayerTime:displayTime:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                5: {
                    "type": sel32or64(
                        b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssLLLssss}QQ}",
                        b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                    ),
                    "type_modifier": b"n",
                }
            },
        },
    )
    r(
        b"CAOpenGLLayer",
        b"drawInCGLContext:pixelFormat:forLayerTime:displayTime:",
        {
            "arguments": {
                5: {
                    "type": sel32or64(
                        b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssLLLssss}QQ}",
                        b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                    ),
                    "type_modifier": b"n",
                }
            }
        },
    )
    r(b"CAOpenGLLayer", b"isAsynchronous", {"retval": {"type": b"Z"}})
    r(b"CAOpenGLLayer", b"setAsynchronous:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"CAOpenGLLayer",
        b"setWantsExtendedDynamicRangeContent:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(b"CAOpenGLLayer", b"wantsExtendedDynamicRangeContent", {"retval": {"type": "Z"}})
    r(b"CAPropertyAnimation", b"isAdditive", {"retval": {"type": b"Z"}})
    r(b"CAPropertyAnimation", b"isCumulative", {"retval": {"type": b"Z"}})
    r(b"CAPropertyAnimation", b"setAdditive:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CAPropertyAnimation", b"setCumulative:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"CARenderer",
        b"beginFrameAtTime:timeStamp:",
        {
            "arguments": {
                3: {
                    "type": sel32or64(
                        b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssLLLssss}QQ}",
                        b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                    ),
                    "type_modifier": b"n",
                }
            }
        },
    )
    r(
        b"CARenderer",
        b"rendererWithCGLContext:options:",
        {"arguments": {2: {"type": "^{_CGLContextObject=}"}}},
    )
    r(b"CAReplicatorLayer", b"preservesDepth", {"retval": {"type": b"Z"}})
    r(b"CAReplicatorLayer", b"setPreservesDepth:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CATextLayer", b"allowsFontSubpixelQuantization", {"retval": {"type": "Z"}})
    r(b"CATextLayer", b"font", {"retval": {"type": b"@"}})
    r(b"CATextLayer", b"isWrapped", {"retval": {"type": b"Z"}})
    r(
        b"CATextLayer",
        b"setAllowsFontSubpixelQuantization:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(b"CATextLayer", b"setFont:", {"arguments": {2: {"type": b"@"}}})
    r(b"CATextLayer", b"setWrapped:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"CATransaction",
        b"completionBlock",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {0: {"type": b"^v"}},
                }
            }
        },
    )
    r(b"CATransaction", b"disableActions", {"retval": {"type": b"Z"}})
    r(
        b"CATransaction",
        b"setCompletionBlock:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}},
                    }
                }
            }
        },
    )
    r(b"CATransaction", b"setDisableActions:", {"arguments": {2: {"type": b"Z"}}})
    r(b"CIAztecCodeDescriptor", b"isCompact", {"retval": {"type": "Z"}})
    r(b"CIColor", b"components", {"retval": {"c_array_of_variable_length": True}})
    r(
        b"CIContext",
        b"createCGImage:fromRect:format:colorSpace:deferred:",
        {"retval": {"already_cfretained": True}, "arguments": {6: {"type": "Z"}}},
    )
    r(
        b"CIContext",
        b"createCGLayerWithSize:info:",
        {"retval": {"already_cfretained": True}},
    )
    r(
        b"CIContext",
        b"prepareRender:fromRect:toDestination:atPoint:error:",
        {"retval": {"type": "Z"}, "arguments": {6: {"type_modifier": b"o"}}},
    )
    r(
        b"CIContext",
        b"render:toBitmap:rowBytes:bounds:format:colorSpace:",
        {"arguments": {3: {"type_modifier": b"o", "c_array_of_variable_length": True}}},
    )
    r(
        b"CIContext",
        b"startTaskToClear:error:",
        {"arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"CIContext",
        b"startTaskToRender:fromRect:toDestination:atPoint:error:",
        {"arguments": {6: {"type_modifier": b"o"}}},
    )
    r(
        b"CIContext",
        b"startTaskToRender:toDestination:error:",
        {"arguments": {4: {"type_modifier": b"o"}}},
    )
    r(
        b"CIContext",
        b"writeHEIFRepresentationOfImage:toURL:format:colorSpace:options:error:",
        {"retval": {"type": "Z"}, "arguments": {7: {"type_modifier": b"o"}}},
    )
    r(
        b"CIContext",
        b"writeJPEGRepresentationOfImage:toURL:colorSpace:options:error:",
        {"retval": {"type": "Z"}, "arguments": {6: {"type_modifier": b"o"}}},
    )
    r(
        b"CIContext",
        b"writePNGRepresentationOfImage:toURL:format:colorSpace:options:error:",
        {"retval": {"type": "Z"}, "arguments": {7: {"type_modifier": b"o"}}},
    )
    r(b"CIFaceFeature", b"hasFaceAngle", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"hasLeftEyePosition", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"hasMouthPosition", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"hasRightEyePosition", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"hasSmile", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"hasTrackingFrameCount", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"hasTrackingID", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"leftEyeClosed", {"retval": {"type": b"Z"}})
    r(b"CIFaceFeature", b"rightEyeClosed", {"retval": {"type": b"Z"}})
    r(b"CIFilter", b"apply:", {"c_array_delimited_by_null": True, "variadic": True})
    r(
        b"CIFilter",
        b"filterArrayFromSerializedXMP:inputImageExtent:error:",
        {"arguments": {4: {"type_modifier": b"o"}}},
    )
    r(
        b"CIFilter",
        b"filterWithName:keysAndValues:",
        {"c_array_delimited_by_null": True, "variadic": True},
    )
    r(b"CIFilter", b"isEnabled", {"retval": {"type": b"Z"}})
    r(b"CIFilter", b"setEnabled:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"CIFilterGenerator",
        b"writeToURL:atomically:",
        {"retval": {"type": b"Z"}, "arguments": {3: {"type": b"Z"}}},
    )
    r(b"CIFilterShape", b"transformBy:interior:", {"arguments": {3: {"type": b"Z"}}})
    r(
        b"CIImage",
        b"imageByApplyingTransform:highQualityDownsample:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(
        b"CIImage",
        b"imageWithTexture:size:flipped:colorSpace:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"CIImage",
        b"imageWithTexture:size:flipped:options:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"CIImage",
        b"initWithTexture:size:flipped:colorSpace:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"CIImage",
        b"initWithTexture:size:flipped:options:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"CIImage",
        b"writeHEIFRepresentationOfImage:toURL:format:colorSpace:options:error:",
        {"retval": {"type": "Z"}, "arguments": {7: {"type_modifier": b"o"}}},
    )
    r(
        b"CIImage",
        b"writeJPEGRepresentationOfImage:toURL:colorSpace:options:error:",
        {"retval": {"type": "Z"}, "arguments": {6: {"type_modifier": b"o"}}},
    )
    r(
        b"CIImage",
        b"writePNGRepresentationOfImage:toURL:format:colorSpace:options:error:",
        {"retval": {"type": "Z"}, "arguments": {7: {"type_modifier": b"o"}}},
    )
    r(
        b"CIImageProcessorKernel",
        b"applyWithExtent:inputs:arguments:error:",
        {"arguments": {5: {"type_modifier": b"o"}}},
    )
    r(
        b"CIImageProcessorKernel",
        b"processWithInputs:arguments:output:error:",
        {"retval": {"type": "Z"}, "arguments": {5: {"type_modifier": b"o"}}},
    )
    r(
        b"CIKernel",
        b"applyWithExtent:roiCallback:arguments:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {
                            "type": sel32or64(
                                b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                                b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                            )
                        },
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"i"},
                            2: {
                                "type": sel32or64(
                                    b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                                )
                            },
                        },
                    }
                }
            }
        },
    )
    r(
        b"CIKernel",
        b"kernelWithFunctionName:fromMetalLibraryData:error:",
        {"arguments": {4: {"type_modifier": b"o"}}},
    )
    r(
        b"CIKernel",
        b"kernelWithFunctionName:fromMetalLibraryData:outputPixelFormat:error:",
        {"arguments": {5: {"type_modifier": b"o"}}},
    )
    r(
        b"CIKernel",
        b"setROISelector:",
        {
            "arguments": {
                2: {
                    "sel_of_type": sel32or64(
                        b"{CGRect={CGPoint=ff}{CGSize=ff}}@:i{CGRect={CGPoint=ff}{CGSize=ff}}@",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}@:i{CGRect={CGPoint=dd}{CGSize=dd}}@",
                    )
                }
            }
        },
    )
    r(b"CIPDF417CodeDescriptor", b"isCompact", {"retval": {"type": "Z"}})
    r(
        b"CIPlugIn",
        b"loadPlugIn:allowExecutableCode:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"CIPlugIn",
        b"loadPlugIn:allowNonExecutable:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"CIRenderDestination",
        b"blendsInDestinationColorSpace",
        {"retval": {"type": "Z"}},
    )
    r(
        b"CIRenderDestination",
        b"initWithWidth:height:pixelFormat:commandBuffer:mtlTextureProvider:",
        {
            "arguments": {
                6: {
                    "callable": {
                        "retval": {"type": b"@"},
                        "arguments": {0: {"type": b"^v"}},
                    }
                }
            }
        },
    )
    r(b"CIRenderDestination", b"isClamped", {"retval": {"type": "Z"}})
    r(b"CIRenderDestination", b"isDithered", {"retval": {"type": "Z"}})
    r(b"CIRenderDestination", b"isFlipped", {"retval": {"type": "Z"}})
    r(
        b"CIRenderDestination",
        b"setBlendsInDestinationColorSpace:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(b"CIRenderDestination", b"setClamped:", {"arguments": {2: {"type": "Z"}}})
    r(b"CIRenderDestination", b"setDithered:", {"arguments": {2: {"type": "Z"}}})
    r(b"CIRenderDestination", b"setFlipped:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"CIRenderTask",
        b"waitUntilCompletedAndReturnError:",
        {"arguments": {2: {"type_modifier": b"o"}}},
    )
    r(
        b"CISampler",
        b"initWithImage:keysAndValues:",
        {"c_array_delimited_by_null": True, "variadic": True},
    )
    r(
        b"CISampler",
        b"samplerWithImage:keysAndValues:",
        {"c_array_delimited_by_null": True, "variadic": True},
    )
    r(
        b"CIVector",
        b"initWithValues:count:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"CIVector",
        b"vectorWithValues:count:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"CIWarpKernel",
        b"applyWithExtent:roiCallback:inputImage:arguments:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {
                            "type": sel32or64(
                                b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                                b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                            )
                        },
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"i"},
                            2: {
                                "type": sel32or64(
                                    b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                                )
                            },
                        },
                    }
                }
            }
        },
    )
    r(b"NSObject", b"EV", {"retval": {"type": "f"}})
    r(b"NSObject", b"NRNoiseLevel", {"retval": {"type": "f"}})
    r(b"NSObject", b"NRSharpness", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"actionForLayer:forKey:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(b"NSObject", b"acuteAngle", {"retval": {"type": "f"}})
    r(b"NSObject", b"alwaysSpecifyCompaction", {"retval": {"type": "f"}})
    r(b"NSObject", b"amount", {"retval": {"type": "f"}})
    r(b"NSObject", b"angle", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"animationDidStart:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"animationDidStop:finished:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"Z"}},
        },
    )
    r(b"NSObject", b"aspectRatio", {"retval": {"type": "f"}})
    r(b"NSObject", b"autoreverses", {"required": True, "retval": {"type": b"Z"}})
    r(b"NSObject", b"barOffset", {"retval": {"type": "f"}})
    r(b"NSObject", b"barcodeHeight", {"retval": {"type": "f"}})
    r(b"NSObject", b"barcodeWidth", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"baseAddress",
        {"retval": {"type": "^v", "c_array_of_variable_length": True}},
    )
    r(b"NSObject", b"beginTime", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"bias", {"retval": {"type": "f"}})
    r(b"NSObject", b"bottomHeight", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"bottomLeft",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"bottomRight",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(b"NSObject", b"brightness", {"retval": {"type": "f"}})
    r(b"NSObject", b"bytesPerRow", {"retval": {"type": "L"}})
    r(
        b"NSObject",
        b"center",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(b"NSObject", b"closeness1", {"retval": {"type": "f"}})
    r(b"NSObject", b"closeness2", {"retval": {"type": "f"}})
    r(b"NSObject", b"closeness3", {"retval": {"type": "f"}})
    r(b"NSObject", b"compactStyle", {"retval": {"type": "f"}})
    r(b"NSObject", b"compactionMode", {"retval": {"type": "f"}})
    r(b"NSObject", b"compression", {"retval": {"type": "f"}})
    r(b"NSObject", b"concentration", {"retval": {"type": "f"}})
    r(b"NSObject", b"contrast", {"retval": {"type": "f"}})
    r(b"NSObject", b"contrast1", {"retval": {"type": "f"}})
    r(b"NSObject", b"contrast2", {"retval": {"type": "f"}})
    r(b"NSObject", b"contrast3", {"retval": {"type": "f"}})
    r(b"NSObject", b"correctionLevel", {"retval": {"type": "f"}})
    r(b"NSObject", b"count", {"retval": {"type": "q"}})
    r(b"NSObject", b"crop", {"retval": {"type": "B"}})
    r(b"NSObject", b"crossAngle", {"retval": {"type": "f"}})
    r(b"NSObject", b"crossOpacity", {"retval": {"type": "f"}})
    r(b"NSObject", b"crossScale", {"retval": {"type": "f"}})
    r(b"NSObject", b"crossWidth", {"retval": {"type": "f"}})
    r(b"NSObject", b"cubeDimension", {"retval": {"type": "f"}})
    r(b"NSObject", b"dataColumns", {"retval": {"type": "f"}})
    r(b"NSObject", b"decay", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"displayLayer:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"dither", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"drawLayer:inContext:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^{CGContext=}"}},
        },
    )
    r(b"NSObject", b"duration", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"edgeIntensity", {"retval": {"type": "f"}})
    r(b"NSObject", b"epsilon", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"extent",
        {
            "retval": {
                "type": sel32or64(
                    b"{CGRect={CGPoint=ff}{CGSize=ff}}",
                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                )
            }
        },
    )
    r(b"NSObject", b"fadeThreshold", {"retval": {"type": "f"}})
    r(b"NSObject", b"falloff", {"retval": {"type": "f"}})
    r(b"NSObject", b"fillMode", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"filterWithName:",
        {"required": True, "retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"focalLength", {"retval": {"type": "f"}})
    r(b"NSObject", b"foldShadowAmount", {"retval": {"type": "f"}})
    r(b"NSObject", b"fontSize", {"retval": {"type": "f"}})
    r(b"NSObject", b"format", {"retval": {"type": sel32or64(b"i", b"q")}})
    r(b"NSObject", b"grayComponentReplacement", {"retval": {"type": "f"}})
    r(b"NSObject", b"haloOverlap", {"retval": {"type": "f"}})
    r(b"NSObject", b"haloRadius", {"retval": {"type": "f"}})
    r(b"NSObject", b"haloWidth", {"retval": {"type": "f"}})
    r(b"NSObject", b"headIndex", {"retval": {"type": "f"}})
    r(b"NSObject", b"height", {"retval": {"type": "f"}})
    r(b"NSObject", b"highlightAmount", {"retval": {"type": "f"}})
    r(b"NSObject", b"intensity", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"invalidateLayoutOfLayer:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"layer", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"layerWillDraw:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"layers", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"layoutSublayersOfLayer:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"levels", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"load:",
        {"required": True, "retval": {"type": b"Z"}, "arguments": {2: {"type": b"^v"}}},
    )
    r(b"NSObject", b"lumaSigma", {"retval": {"type": "f"}})
    r(b"NSObject", b"maxHeight", {"retval": {"type": "f"}})
    r(b"NSObject", b"maxStriationRadius", {"retval": {"type": "f"}})
    r(b"NSObject", b"maxWidth", {"retval": {"type": "f"}})
    r(b"NSObject", b"minHeight", {"retval": {"type": "f"}})
    r(b"NSObject", b"minWidth", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"neutral",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(b"NSObject", b"noiseLevel", {"retval": {"type": "f"}})
    r(b"NSObject", b"numberOfFolds", {"retval": {"type": "f"}})
    r(b"NSObject", b"opacity", {"retval": {"type": "f"}})
    r(b"NSObject", b"parameterB", {"retval": {"type": "f"}})
    r(b"NSObject", b"parameterC", {"retval": {"type": "f"}})
    r(b"NSObject", b"perceptual", {"retval": {"type": "B"}})
    r(b"NSObject", b"pitch", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"point",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"point0",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"point1",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"point2",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"point3",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"point4",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(b"NSObject", b"power", {"retval": {"type": "f"}})
    r(b"NSObject", b"preferredAspectRatio", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"preferredSizeOfLayer:",
        {
            "required": False,
            "retval": {"type": sel32or64(b"{_NSSize=ff}", b"{CGSize=dd}")},
            "arguments": {2: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"provideImageData:bytesPerRow:origin::size::userInfo:",
        {
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "type": b"^v",
                    "type_modifier": b"o",
                    "c_array_of_variable_length": True,
                },
                3: {"type": sel32or64(b"L", b"Q")},
                4: {"type": sel32or64(b"L", b"Q")},
                5: {"type": sel32or64(b"L", b"Q")},
                6: {"type": sel32or64(b"L", b"Q")},
                7: {"type": sel32or64(b"L", b"Q")},
                8: {"type": b"@"},
            },
        },
    )
    r(b"NSObject", b"quietSpace", {"retval": {"type": "f"}})
    r(b"NSObject", b"radius", {"retval": {"type": "f"}})
    r(b"NSObject", b"radius0", {"retval": {"type": "f"}})
    r(b"NSObject", b"radius1", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"region",
        {
            "retval": {
                "type": sel32or64(
                    b"{CGRect={CGPoint=ff}{CGSize=ff}}",
                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                )
            }
        },
    )
    r(b"NSObject", b"repeatCount", {"required": True, "retval": {"type": b"f"}})
    r(b"NSObject", b"repeatDuration", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"ringAmount", {"retval": {"type": "f"}})
    r(b"NSObject", b"ringSize", {"retval": {"type": "f"}})
    r(b"NSObject", b"roll", {"retval": {"type": "f"}})
    r(b"NSObject", b"rotation", {"retval": {"type": "f"}})
    r(b"NSObject", b"rows", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"runActionForKey:object:arguments:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(b"NSObject", b"saturation", {"retval": {"type": "f"}})
    r(b"NSObject", b"scaleFactor", {"retval": {"type": "f"}})
    r(b"NSObject", b"setAcuteAngle:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setAlwaysSpecifyCompaction:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setAmount:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setAngle:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setAspectRatio:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setAutoreverses:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"Z"}}},
    )
    r(b"NSObject", b"setBarOffset:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setBarcodeHeight:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setBarcodeWidth:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setBeginTime:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"d"}}},
    )
    r(b"NSObject", b"setBias:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setBottomHeight:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setBottomLeft:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setBottomRight:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(b"NSObject", b"setBrightness:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setCenter:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(b"NSObject", b"setCloseness1:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCloseness2:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCloseness3:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCompactStyle:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCompactionMode:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCompression:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setConcentration:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setContrast1:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setContrast2:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setContrast3:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setContrast:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCorrectionLevel:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCount:", {"arguments": {2: {"type": "q"}}})
    r(b"NSObject", b"setCrop:", {"arguments": {2: {"type": "B"}}})
    r(b"NSObject", b"setCrossAngle:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCrossOpacity:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCrossScale:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCrossWidth:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setCubeDimension:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setDataColumns:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setDecay:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setDither:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setDuration:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"d"}}},
    )
    r(b"NSObject", b"setEV:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setEdgeIntensity:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setEpsilon:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setExtent:",
        {
            "arguments": {
                2: {
                    "type": sel32or64(
                        b"{CGRect={CGPoint=ff}{CGSize=ff}}",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                }
            }
        },
    )
    r(b"NSObject", b"setFadeThreshold:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setFalloff:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setFillMode:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"setFocalLength:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setFoldShadowAmount:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setFontSize:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setGrayComponentReplacement:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setHaloOverlap:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setHaloRadius:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setHaloWidth:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setHeadIndex:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setHeight:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setHighlightAmount:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setIntensity:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setLayers:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setLevels:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setLumaSigma:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setMaxHeight:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setMaxStriationRadius:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setMaxWidth:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setMinHeight:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setMinWidth:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setNRNoiseLevel:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setNRSharpness:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setNeutral:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(b"NSObject", b"setNoiseLevel:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setNumberOfFolds:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setOpacity:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setParameterB:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setParameterC:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setPerceptual:", {"arguments": {2: {"type": "B"}}})
    r(b"NSObject", b"setPitch:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setPoint0:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setPoint1:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setPoint2:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setPoint3:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setPoint4:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setPoint:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(b"NSObject", b"setPower:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setPreferredAspectRatio:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setQuietSpace:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setRadius0:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setRadius1:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setRadius:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setRepeatCount:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"f"}}},
    )
    r(
        b"NSObject",
        b"setRepeatDuration:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"d"}}},
    )
    r(b"NSObject", b"setRingAmount:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setRingSize:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setRoll:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setRotation:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setRows:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setSaturation:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setScaleFactor:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setShadowAmount:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setShadowDensity:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setShadowExtent:",
        {
            "arguments": {
                2: {
                    "type": sel32or64(
                        b"{CGRect={CGPoint=ff}{CGSize=ff}}",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                }
            }
        },
    )
    r(
        b"NSObject",
        b"setShadowOffset:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(b"NSObject", b"setShadowRadius:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setShadowSize:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setSharpness:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setSize:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setSoftmaxNormalization:", {"arguments": {2: {"type": "B"}}})
    r(b"NSObject", b"setSoftness:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setSpatialSigma:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setSpeed:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"f"}}},
    )
    r(b"NSObject", b"setStriationContrast:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setStriationStrength:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setSunRadius:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setTargetNeutral:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(b"NSObject", b"setThreshold:", {"arguments": {2: {"type": "f"}}})
    r(
        b"NSObject",
        b"setTimeOffset:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"d"}}},
    )
    r(
        b"NSObject",
        b"setTopLeft:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setTopRight:",
        {"arguments": {2: {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}}},
    )
    r(
        b"NSObject",
        b"setTransform:",
        {
            "arguments": {
                2: {
                    "type": sel32or64(
                        b"{CGAffineTransform=ffffff}", b"{CGAffineTransform=dddddd}"
                    )
                }
            }
        },
    )
    r(b"NSObject", b"setUnderColorRemoval:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setUnsharpMaskIntensity:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setUnsharpMaskRadius:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setValue:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setWidth:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"setYaw:", {"arguments": {2: {"type": "f"}}})
    r(b"NSObject", b"shadowAmount", {"retval": {"type": "f"}})
    r(b"NSObject", b"shadowDensity", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"shadowExtent",
        {
            "retval": {
                "type": sel32or64(
                    b"{CGRect={CGPoint=ff}{CGSize=ff}}",
                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                )
            }
        },
    )
    r(
        b"NSObject",
        b"shadowOffset",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(b"NSObject", b"shadowRadius", {"retval": {"type": "f"}})
    r(b"NSObject", b"shadowSize", {"retval": {"type": "f"}})
    r(b"NSObject", b"sharpness", {"retval": {"type": "f"}})
    r(b"NSObject", b"size", {"retval": {"type": "f"}})
    r(b"NSObject", b"softmaxNormalization", {"retval": {"type": "B"}})
    r(b"NSObject", b"softness", {"retval": {"type": "f"}})
    r(b"NSObject", b"spatialSigma", {"retval": {"type": "f"}})
    r(b"NSObject", b"speed", {"required": True, "retval": {"type": b"f"}})
    r(b"NSObject", b"striationContrast", {"retval": {"type": "f"}})
    r(b"NSObject", b"striationStrength", {"retval": {"type": "f"}})
    r(b"NSObject", b"sunRadius", {"retval": {"type": "f"}})
    r(
        b"NSObject",
        b"targetNeutral",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(b"NSObject", b"texture", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"threshold", {"retval": {"type": "f"}})
    r(b"NSObject", b"timeOffset", {"required": True, "retval": {"type": b"d"}})
    r(
        b"NSObject",
        b"topLeft",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"topRight",
        {"retval": {"type": sel32or64(b"{CGPoint=ff}", b"{CGPoint=dd}")}},
    )
    r(
        b"NSObject",
        b"transform",
        {
            "retval": {
                "type": sel32or64(
                    b"{CGAffineTransform=ffffff}", b"{CGAffineTransform=dddddd}"
                )
            }
        },
    )
    r(b"NSObject", b"underColorRemoval", {"retval": {"type": "f"}})
    r(b"NSObject", b"unsharpMaskIntensity", {"retval": {"type": "f"}})
    r(b"NSObject", b"unsharpMaskRadius", {"retval": {"type": "f"}})
    r(b"NSObject", b"value", {"retval": {"type": "f"}})
    r(b"NSObject", b"width", {"retval": {"type": "f"}})
    r(b"NSObject", b"yaw", {"retval": {"type": "f"}})
finally:
    objc._updatingMetadata(False)
protocols = {
    "CAAnimationDelegate": objc.informal_protocol(
        "CAAnimationDelegate",
        [
            objc.selector(None, b"animationDidStart:", b"v@:@", isRequired=False),
            objc.selector(
                None, b"animationDidStop:finished:", b"v@:@Z", isRequired=False
            ),
        ],
    ),
    "CALayerDelegate": objc.informal_protocol(
        "CALayerDelegate",
        [
            objc.selector(
                None, b"drawLayer:inContext:", b"v@:@^{CGContext=}", isRequired=False
            ),
            objc.selector(None, b"actionForLayer:forKey:", b"@@:@@", isRequired=False),
            objc.selector(None, b"displayLayer:", b"v@:@", isRequired=False),
            objc.selector(None, b"layoutSublayersOfLayer:", b"v@:@", isRequired=False),
        ],
    ),
    "CIImageProvider": objc.informal_protocol(
        "CIImageProvider",
        [
            objc.selector(
                None,
                b"provideImageData:bytesPerRow:origin::size::userInfo:",
                sel32or64(b"v@:^vLLLLL@", b"v@:^vQQQQQ@"),
                isRequired=False,
            )
        ],
    ),
    "CALayoutManager": objc.informal_protocol(
        "CALayoutManager",
        [
            objc.selector(
                None,
                b"preferredSizeOfLayer:",
                sel32or64(b"{CGSize=ff}@:@", b"{CGSize=dd}@:@"),
                isRequired=False,
            ),
            objc.selector(None, b"layoutSublayersOfLayer:", b"v@:@", isRequired=False),
            objc.selector(None, b"invalidateLayoutOfLayer:", b"v@:@", isRequired=False),
        ],
    ),
}
expressions = {}

# END OF FILE



import random
from math import pi as PI

import Cocoa
import objc
import Quartz
from objc import super


class DemoView(Cocoa.NSView):
    _demoNumber = objc.ivar(type=objc._C_INT)

    def initWithFrame_(self, frameRect):
        self = super(DemoView, self).initWithFrame_(frameRect)
        if self is None:
            return None

        self._demoNumber = 0
        return self

    def drawRect_(self, rect):
        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()

        Quartz.CGContextSetGrayFillColor(context, 1.0, 1.0)
        Quartz.CGContextFillRect(context, rect)

        if self._demoNumber == 0:
            rectangles(context, rect)

        elif self._demoNumber == 1:
            circles(context, rect)

        elif self._demoNumber == 2:
            bezierPaths(context, rect)

        elif self._demoNumber == 3:
            circleClipping(context, rect)

        else:
            Cocoa.NSLog("Invalid demo number.")

    def setDemoNumber_(self, number):
        self._demoNumber = number


# The various demo functions


def setRandomFillColor(context):
    Quartz.CGContextSetRGBFillColor(
        context,
        random.uniform(0, 1),
        random.uniform(0, 1),
        random.uniform(0, 1),
        random.uniform(0, 1),
    )


def setRandomStrokeColor(context):
    Quartz.CGContextSetRGBStrokeColor(
        context,
        random.uniform(0, 1),
        random.uniform(0, 1),
        random.uniform(0, 1),
        random.uniform(0, 1),
    )


def randomPointInRect(rect):
    return Quartz.CGPoint(
        x=random.uniform(Quartz.CGRectGetMinX(rect), Quartz.CGRectGetMaxX(rect)),
        y=random.uniform(Quartz.CGRectGetMinY(rect), Quartz.CGRectGetMaxY(rect)),
    )


def randomRectInRect(rect):
    p = randomPointInRect(rect)
    q = randomPointInRect(rect)
    return Quartz.CGRectMake(p.x, p.y, q.x - p.x, q.y - p.y)


def rectangles(context, rect):
    # Draw random rectangles (some stroked, some filled).

    for k in range(20):
        if k % 2 == 0:
            setRandomFillColor(context)
            Quartz.CGContextFillRect(context, randomRectInRect(rect))

        else:
            setRandomStrokeColor(context)
            Quartz.CGContextSetLineWidth(context, 2 + random.randint(0, 10))
            Quartz.CGContextStrokeRect(context, randomRectInRect(rect))


def circles(context, rect):
    # Draw random circles (some stroked, some filled).

    for k in range(20):
        r = randomRectInRect(rect)
        w = Quartz.CGRectGetWidth(r)
        h = Quartz.CGRectGetHeight(r)
        Quartz.CGContextBeginPath(context)

        if w < h:
            v = w
        else:
            v = h

        Quartz.CGContextAddArc(
            context,
            Quartz.CGRectGetMidX(r),
            Quartz.CGRectGetMidY(r),
            v,
            0,
            2 * PI,
            False,
        )
        Quartz.CGContextClosePath(context)

        if k % 2 == 0:
            setRandomFillColor(context)
            Quartz.CGContextFillPath(context)

        else:
            setRandomStrokeColor(context)
            Quartz.CGContextSetLineWidth(context, 2 + random.randint(0, 10))
            Quartz.CGContextStrokePath(context)


def bezierPaths(context, rect):
    for k in range(20):
        numberOfSegments = 1 + random.randint(0, 8)
        Quartz.CGContextBeginPath(context)
        p = randomPointInRect(rect)
        Quartz.CGContextMoveToPoint(context, p.x, p.y)
        for j in range(numberOfSegments):
            p = randomPointInRect(rect)

            if j % 2 == 0:
                Quartz.CGContextAddLineToPoint(context, p.x, p.y)

            else:
                c1 = randomPointInRect(rect)
                c2 = randomPointInRect(rect)
                Quartz.CGContextAddCurveToPoint(
                    context, c1.x, c1.y, c2.x, c2.y, p.x, p.y
                )

        if k % 2 == 0:
            setRandomFillColor(context)
            Quartz.CGContextClosePath(context)
            Quartz.CGContextFillPath(context)

        else:
            setRandomStrokeColor(context)
            Quartz.CGContextSetLineWidth(context, 2 + random.randint(0, 10))
            Quartz.CGContextStrokePath(context)


def circleClipping(context, rect):
    # Draw a random path through a circular clip.

    w = Quartz.CGRectGetWidth(rect)
    h = Quartz.CGRectGetHeight(rect)
    Quartz.CGContextBeginPath(context)
    if w < h:
        v = w
    else:
        v = h
    Quartz.CGContextAddArc(
        context,
        Quartz.CGRectGetMidX(rect),
        Quartz.CGRectGetMidY(rect),
        v / 2,
        0,
        2 * PI,
        False,
    )
    Quartz.CGContextClosePath(context)
    Quartz.CGContextClip(context)

    # Draw something into the clip.
    bezierPaths(context, rect)

    # Draw a clip path on top as a black stroked circle.
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextAddArc(
        context,
        Quartz.CGRectGetMidX(rect),
        Quartz.CGRectGetMidY(rect),
        v / 2,
        0,
        2 * PI,
        False,
    )
    Quartz.CGContextClosePath(context)
    Quartz.CGContextSetLineWidth(context, 1)
    Quartz.CGContextSetRGBStrokeColor(context, 0, 0, 0, 1)
    Quartz.CGContextStrokePath(context)



import random

import Cocoa
import objc


class NSColor(objc.Category(Cocoa.NSColor)):
    @classmethod
    def randomColor(self):
        return Cocoa.NSColor.colorWithCalibratedRed_green_blue_alpha_(
            random.uniform(0, 1), random.uniform(0, 1), random.uniform(0, 1), 1
        )


def makeRandomPointInRect(rect):
    return Cocoa.NSPoint(
        x=random.uniform(Cocoa.NSMinX(rect), Cocoa.NSMaxX(rect)),
        y=random.uniform(Cocoa.NSMinY(rect), Cocoa.NSMaxY(rect)),
    )



import math

import Quartz


def doEgg(context):
    p0 = Quartz.CGPoint(0, 0)
    p1 = Quartz.CGPoint(0, 200)
    c1 = Quartz.CGPoint(140, 5)
    c2 = Quartz.CGPoint(80, 198)

    Quartz.CGContextTranslateCTM(context, 100, 5)
    Quartz.CGContextBeginPath(context)

    Quartz.CGContextMoveToPoint(context, p0.x, p0.y)
    # Create the Bezier path segment for the right side of the egg.
    Quartz.CGContextAddCurveToPoint(context, c1.x, c1.y, c2.x, c2.y, p1.x, p1.y)
    # Create the Bezier path segment for the left side of the egg.
    Quartz.CGContextAddCurveToPoint(context, -c2.x, c2.y, -c1.x, c1.y, p0.x, p0.y)
    Quartz.CGContextClosePath(context)
    Quartz.CGContextSetLineWidth(context, 2)
    Quartz.CGContextDrawPath(context, Quartz.kCGPathStroke)


def addRoundedRectToPath(context, rect, ovalWidth, ovalHeight):
    # If either ovalWidth or ovalHeight is 0, draw a regular rectangle.
    if ovalWidth == 0 or ovalHeight == 0:
        Quartz.CGContextAddRect(context, rect)
    else:
        Quartz.CGContextSaveGState(context)
        if 1:
            # Translate to lower-left corner of rectangle.
            Quartz.CGContextTranslateCTM(
                context, Quartz.CGRectGetMinX(rect), Quartz.CGRectGetMinY(rect)
            )
            # Scale by the oval width and height so that
            # each rounded corner is 0.5 units in radius.
            Quartz.CGContextScaleCTM(context, ovalWidth, ovalHeight)
            # Unscale the rectangle width by the amount of the X scaling.
            fw = Quartz.CGRectGetWidth(rect) / ovalWidth
            # Unscale the rectangle height by the amount of the Y scaling.
            fh = Quartz.CGRectGetHeight(rect) / ovalHeight
            # Start at the right edge of the rect, at the midpoint in Y.
            Quartz.CGContextMoveToPoint(context, fw, fh / 2)
            # Segment 1
            Quartz.CGContextAddArcToPoint(context, fw, fh, fw / 2, fh, 0.5)
            # Segment 2
            Quartz.CGContextAddArcToPoint(context, 0, fh, 0, fh / 2, 0.5)
            # Segment 3
            Quartz.CGContextAddArcToPoint(context, 0, 0, fw / 2, 0, 0.5)
            # Segment 4
            Quartz.CGContextAddArcToPoint(context, fw, 0, fw, fh / 2, 0.5)
            # Closing the path adds the last segment.
            Quartz.CGContextClosePath(context)
        Quartz.CGContextRestoreGState(context)


def doRoundedRects(context):
    rect = Quartz.CGRectMake(10, 10, 210, 150)
    ovalWidth = 100
    ovalHeight = 100
    Quartz.CGContextSetLineWidth(context, 2.0)
    Quartz.CGContextBeginPath(context)
    addRoundedRectToPath(context, rect, ovalWidth, ovalHeight)
    Quartz.CGContextSetRGBStrokeColor(context, 1, 0, 0, 1)
    Quartz.CGContextDrawPath(context, Quartz.kCGPathStroke)


def doStrokeWithCTM(context):
    Quartz.CGContextTranslateCTM(context, 150.0, 180.0)
    Quartz.CGContextSetLineWidth(context, 10)
    # Draw ellipse 1 with a uniform stroke.
    Quartz.CGContextSaveGState(context)
    if 1:
        # Scale the CTM so the circular arc will be elliptical.
        Quartz.CGContextScaleCTM(context, 2, 1)
        Quartz.CGContextBeginPath(context)
        # Create an arc that is a circle.
        Quartz.CGContextAddArc(context, 0.0, 0.0, 45.0, 0.0, 2 * math.pi, 0)
        # Restore the context parameters prior to stroking the path.
        # CGContextRestoreGState does not affect the path in the context.
    Quartz.CGContextRestoreGState(context)
    Quartz.CGContextStrokePath(context)

    # *** was 0, -120
    Quartz.CGContextTranslateCTM(context, 220.0, 0.0)
    # Draw ellipse 2 with non-uniform stroke.
    Quartz.CGContextSaveGState(context)
    if 1:
        # Scale the CTM so the circular arc will be elliptical.
        Quartz.CGContextScaleCTM(context, 2, 1)
        Quartz.CGContextBeginPath(context)
        # Create an arc that is a circle.
        Quartz.CGContextAddArc(context, 0.0, 0.0, 45.0, 0.0, 2 * math.pi, 0)
        # Stroke the path with the scaled coordinate system in effect.
        Quartz.CGContextStrokePath(context)
    Quartz.CGContextRestoreGState(context)


def doRotatedEllipsesWithCGPath(context):
    totreps = 144
    tint = 1.0
    tintIncrement = 1.0 / totreps

    # Create a new transform consisting of a 45 degree rotation.
    theTransform = Quartz.CGAffineTransformMakeRotation(math.pi / 4)
    # Apply a scaling transformation to the transform just created.
    theTransform = Quartz.CGAffineTransformScale(theTransform, 1, 2)
    # Create a mutable CGPath object.
    path = Quartz.CGPathCreateMutable()
    if path is None:
        print("Couldn't create path!")
        return

    # Add a circular arc to the CGPath object, transformed
    # by an affine transform.
    Quartz.CGPathAddArc(path, theTransform, 0.0, 0.0, 45.0, 0.0, 2 * math.pi, False)
    # Close the CGPath object.
    Quartz.CGPathCloseSubpath(path)

    # Place the first ellipse at a good location.
    Quartz.CGContextTranslateCTM(context, 100, 100)
    for _ in range(totreps):
        Quartz.CGContextBeginPath(context)
        # Add the CGPath object to the current path in the context.
        Quartz.CGContextAddPath(context, path)

        # Set the fill color for this instance of the ellipse.
        Quartz.CGContextSetRGBFillColor(context, tint, 0.0, 0.0, 1.0)
        # Filling the path implicitly closes it.
        Quartz.CGContextFillPath(context)
        # Compute the next tint color.
        tint -= tintIncrement
        # Move over for the next ellipse.
        Quartz.CGContextTranslateCTM(context, 1, 0.0)


def alignPointToUserSpace(context, p):
    # Compute the coordinates of the point in device space.
    p = Quartz.CGContextConvertPointToDeviceSpace(context, p)
    # Ensure that coordinates are at exactly the corner
    # of a device pixel.
    p.x = math.floor(p.x)
    p.y = math.floor(p.y)
    # Convert the device aligned coordinate back to user space.
    return Quartz.CGContextConvertPointToUserSpace(context, p)


def alignSizeToUserSpace(context, s):
    # Compute the size in device space.
    s = Quartz.CGContextConvertSizeToDeviceSpace(context, s)
    # Ensure that size is an integer multiple of device pixels.
    s.width = math.floor(s.width)
    s.height = math.floor(s.height)
    # Convert back to user space.
    return Quartz.CGContextConvertSizeToUserSpace(context, s)


def alignRectToUserSpace(context, r):
    # Compute the coordinates of the rectangle in device space.
    r = Quartz.CGContextConvertRectToDeviceSpace(context, r)
    # Ensure that the x and y coordinates are at a pixel corner.
    r.origin.x = math.floor(r.origin.x)
    r.origin.y = math.floor(r.origin.y)
    # Ensure that the width and height are an integer number of
    # device pixels. Note that this produces a width and height
    # that is less than or equal to the original width. Another
    # approach is to use ceil to ensure that the new rectangle
    # encloses the original one.
    r.size.width = math.floor(r.size.width)
    r.size.height = math.floor(r.size.height)

    # Convert back to user space.
    return Quartz.CGContextConvertRectToUserSpace(context, r)


def doPixelAlignedFillAndStroke(context):
    p1 = Quartz.CGPointMake(16.7, 17.8)
    p2 = Quartz.CGPointMake(116.7, 17.8)
    r = Quartz.CGRectMake(16.7, 20.8, 100.6, 100.6)

    Quartz.CGContextSetLineWidth(context, 2)
    Quartz.CGContextSetRGBFillColor(context, 1.0, 0.0, 0.0, 1.0)
    Quartz.CGContextSetRGBStrokeColor(context, 1.0, 0.0, 0.0, 1.0)

    # Unaligned drawing.
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextMoveToPoint(context, p1.x, p1.y)
    Quartz.CGContextAddLineToPoint(context, p2.x, p2.y)
    Quartz.CGContextStrokePath(context)
    Quartz.CGContextFillRect(context, r)

    # Translate to the right before drawing along
    # aligned coordinates.
    Quartz.CGContextTranslateCTM(context, 106, 0)

    # Aligned drawing.

    # Compute the length of the line in user space.
    s = Quartz.CGSizeMake(p2.x - p1.x, p2.y - p1.y)

    Quartz.CGContextBeginPath(context)
    # Align the starting point to a device
    # pixel boundary.
    p1 = alignPointToUserSpace(context, p1)
    # Establish the starting point of the line.
    Quartz.CGContextMoveToPoint(context, p1.x, p1.y)
    # Compute the line length as an integer
    # number of device pixels.
    s = alignSizeToUserSpace(context, s)
    Quartz.CGContextAddLineToPoint(context, p1.x + s.width, p1.y + s.height)
    Quartz.CGContextStrokePath(context)
    # Compute a rect that is aligned to device
    # space with a width that is an integer
    # number of device pixels.
    r = alignRectToUserSpace(context, r)
    Quartz.CGContextFillRect(context, r)



import math

import Cocoa
import objc
import Quartz

ShadowOffsetChanged = "ShadowOffsetChanged"


class ShadowOffsetView(Cocoa.NSView):
    _offset = objc.ivar(type=Quartz.CGSize.__typestr__)
    _scale = objc.ivar(type=objc._C_FLT)

    def scale(self):
        return self._scale

    def setScale_(self, scale):
        self._scale = scale

    def offset(self):
        return Quartz.CGSizeMake(
            self._offset.width * self._scale, self._offset.height * self._scale
        )

    def setOffset_(self, offset):
        offset = Quartz.CGSizeMake(
            offset.width / self._scale, offset.height / self._scale
        )
        if self._offset != offset:
            self._offset = offset
            self.setNeedsDisplay_(True)

    def isOpaque(self):
        return False

    def setOffsetFromPoint_(self, point):
        bounds = self.bounds()
        offset = Quartz.CGSize(
            width=(point.x - Cocoa.NSMidX(bounds)) / (Cocoa.NSWidth(bounds) / 2),
            height=(point.y - Cocoa.NSMidY(bounds)) / (Cocoa.NSHeight(bounds) / 2),
        )
        radius = math.sqrt(offset.width * offset.width + offset.height * offset.height)
        if radius > 1:
            offset.width /= radius
            offset.height /= radius

        if self._offset != offset:
            self._offset = offset
            self.setNeedsDisplay_(True)
            Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_(
                ShadowOffsetChanged, self
            )

    def mouseDown_(self, event):
        point = self.convertPoint_fromView_(event.locationInWindow(), None)
        self.setOffsetFromPoint_(point)

    def mouseDragged_(self, event):
        point = self.convertPoint_fromView_(event.locationInWindow(), None)
        self.setOffsetFromPoint_(point)

    def drawRect_(self, rect):
        bounds = self.bounds()
        x = Cocoa.NSMinX(bounds)
        y = Cocoa.NSMinY(bounds)
        w = Cocoa.NSWidth(bounds)
        h = Cocoa.NSHeight(bounds)
        r = min(w / 2, h / 2)

        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()

        Quartz.CGContextTranslateCTM(context, x + w / 2, y + h / 2)

        Quartz.CGContextAddArc(context, 0, 0, r, 0, math.pi, True)
        Quartz.CGContextClip(context)

        Quartz.CGContextSetGrayFillColor(context, 0.910, 1)
        Quartz.CGContextFillRect(context, Quartz.CGRectMake(-w / 2, -h / 2, w, h))

        Quartz.CGContextAddArc(context, 0, 0, r, 0, 2 * math.pi, True)
        Quartz.CGContextSetGrayStrokeColor(context, 0.616, 1)
        Quartz.CGContextStrokePath(context)

        Quartz.CGContextAddArc(context, 0, -2, r, 0, 2 * math.pi, True)
        Quartz.CGContextSetGrayStrokeColor(context, 0.784, 1)
        Quartz.CGContextStrokePath(context)

        Quartz.CGContextMoveToPoint(context, 0, 0)
        Quartz.CGContextAddLineToPoint(
            context, r * self._offset.width, r * self._offset.height
        )

        Quartz.CGContextSetLineWidth(context, 2)
        Quartz.CGContextSetGrayStrokeColor(context, 0.33, 1)
        Quartz.CGContextStrokePath(context)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
import os

from setuptools import setup

setup(
    name="BasicDrawing",
    app=["main.py"],
    data_files=["English.lproj"]
    + [os.path.join("GraphicsFiles", fn) for fn in os.listdir("GraphicsFiles")],
    setup_requires=["py2app", "pyobjc-framework-Cocoa", "pyobjc-framework-Quartz"],
)



import math

import Cocoa
import Quartz


def doAlphaRects(context):
    # ***** Part 1 *****
    ourRect = Quartz.CGRectMake(0.0, 0.0, 130.0, 100.0)
    numRects = 6
    rotateAngle = 2 * math.pi / numRects
    tintAdjust = 1.0 / numRects

    # ***** Part 2 *****
    Quartz.CGContextTranslateCTM(
        context, 2 * ourRect.size.width, 2 * ourRect.size.height
    )

    # ***** Part 3 *****
    tint = 1.0
    for _ in range(numRects):
        Quartz.CGContextSetRGBFillColor(context, tint, 0.0, 0.0, tint)
        Quartz.CGContextFillRect(context, ourRect)
        Quartz.CGContextRotateCTM(context, rotateAngle)  # cumulative
        tint -= tintAdjust


class MyView(Cocoa.NSView):
    def drawRect_(self, rect):
        nsctx = Cocoa.NSGraphicsContext.currentContext()
        context = nsctx.graphicsPort()

        Quartz.CGContextSetLineWidth(context, 5.0)
        # Draw the coordinate axes.
        Quartz.CGContextBeginPath(context)
        # First draw the x axis.
        Quartz.CGContextMoveToPoint(context, -2000.0, 0.0)
        Quartz.CGContextAddLineToPoint(context, 2000.0, 0.0)
        Quartz.CGContextDrawPath(context, Quartz.kCGPathStroke)
        # Next draw the y axis.
        Quartz.CGContextMoveToPoint(context, 0.0, -2000.0)
        Quartz.CGContextAddLineToPoint(context, 0.0, 2000.0)
        Quartz.CGContextDrawPath(context, Quartz.kCGPathStroke)

        doAlphaRects(context)



# This file is generated by objective.metadata
#
# Last update: Sun Aug  4 21:30:44 2019
#
# flake8: noqa

import sys

import objc

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$QCCompositionAttributeBuiltInKey$QCCompositionAttributeCategoryKey$QCCompositionAttributeCopyrightKey$QCCompositionAttributeDescriptionKey$QCCompositionAttributeHasConsumersKey$QCCompositionAttributeIsTimeDependentKey$QCCompositionAttributeNameKey$QCCompositionCategoryDistortion$QCCompositionCategoryStylize$QCCompositionCategoryUtility$QCCompositionInputAudioPeakKey$QCCompositionInputAudioSpectrumKey$QCCompositionInputDestinationImageKey$QCCompositionInputImageKey$QCCompositionInputPaceKey$QCCompositionInputPreviewModeKey$QCCompositionInputPrimaryColorKey$QCCompositionInputRSSArticleDurationKey$QCCompositionInputRSSFeedURLKey$QCCompositionInputScreenImageKey$QCCompositionInputSecondaryColorKey$QCCompositionInputSourceImageKey$QCCompositionInputTrackInfoKey$QCCompositionInputTrackPositionKey$QCCompositionInputTrackSignalKey$QCCompositionInputXKey$QCCompositionInputYKey$QCCompositionOutputImageKey$QCCompositionOutputWebPageURLKey$QCCompositionPickerPanelDidSelectCompositionNotification$QCCompositionPickerViewDidSelectCompositionNotification$QCCompositionProtocolGraphicAnimation$QCCompositionProtocolGraphicTransition$QCCompositionProtocolImageFilter$QCCompositionProtocolMusicVisualizer$QCCompositionProtocolRSSVisualizer$QCCompositionProtocolScreenSaver$QCCompositionRepositoryDidUpdateNotification$QCPlugInAttributeCategoriesKey$QCPlugInAttributeCopyrightKey$QCPlugInAttributeDescriptionKey$QCPlugInAttributeExamplesKey$QCPlugInAttributeNameKey$QCPlugInExecutionArgumentEventKey$QCPlugInExecutionArgumentMouseLocationKey$QCPlugInPixelFormatARGB8$QCPlugInPixelFormatBGRA8$QCPlugInPixelFormatI8$QCPlugInPixelFormatIf$QCPlugInPixelFormatRGBAf$QCPortAttributeDefaultValueKey$QCPortAttributeMaximumValueKey$QCPortAttributeMenuItemsKey$QCPortAttributeMinimumValueKey$QCPortAttributeNameKey$QCPortAttributeTypeKey$QCPortTypeBoolean$QCPortTypeColor$QCPortTypeImage$QCPortTypeIndex$QCPortTypeNumber$QCPortTypeString$QCPortTypeStructure$QCRendererEventKey$QCRendererMouseLocationKey$QCViewDidStartRenderingNotification$QCViewDidStopRenderingNotification$"""
enums = """$kQCPlugInExecutionModeConsumer@3$kQCPlugInExecutionModeProcessor@2$kQCPlugInExecutionModeProvider@1$kQCPlugInTimeModeIdle@1$kQCPlugInTimeModeNone@0$kQCPlugInTimeModeTimeBase@2$"""
misc.update({})
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"NSObject",
        b"CGLContextObj",
        {"required": True, "retval": {"type": b"^{_CGLContextObject=}"}},
    )
    r(b"NSObject", b"attributes", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"bindTextureRepresentationToCGLContext:textureUnit:normalizeCoordinates:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"^{_CGLContextObject=}"},
                3: {"type": b"I"},
                4: {"type": b"Z"},
            },
        },
    )
    r(
        b"NSObject",
        b"bounds",
        {
            "required": True,
            "retval": {
                "type": sel32or64(
                    b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                )
            },
        },
    )
    r(b"NSObject", b"bufferBaseAddress", {"required": True, "retval": {"type": b"^v"}})
    r(
        b"NSObject",
        b"bufferBytesPerRow",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"bufferColorSpace",
        {"required": True, "retval": {"type": b"^{CGColorSpace=}"}},
    )
    r(b"NSObject", b"bufferPixelFormat", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"bufferPixelsHigh",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"bufferPixelsWide",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"canRenderWithCGLContext:",
        {
            "required": False,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"^{_CGLContextObject=}"}},
        },
    )
    r(
        b"NSObject",
        b"colorSpace",
        {"required": True, "retval": {"type": b"^{CGColorSpace=}"}},
    )
    r(
        b"NSObject",
        b"compositionParameterView:didChangeParameterWithKey:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"compositionParameterView:shouldDisplayParameterWithKey:attributes:",
        {
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"compositionPickerView:didSelectComposition:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"compositionPickerViewDidStartAnimating:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"compositionPickerViewWillStopAnimating:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"compositionURL", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"copyRenderedTextureForCGLContext:pixelFormat:bounds:isFlipped:",
        {
            "required": False,
            "retval": {"type": b"I"},
            "arguments": {
                2: {"type": b"^{_CGLContextObject=}"},
                3: {"type": b"@"},
                4: {
                    "type": sel32or64(
                        b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                },
                5: {"type": b"^Z"},
            },
        },
    )
    r(
        b"NSObject",
        b"imageBounds",
        {
            "required": True,
            "retval": {
                "type": sel32or64(
                    b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                )
            },
        },
    )
    r(
        b"NSObject",
        b"imageColorSpace",
        {"required": True, "retval": {"type": b"^{CGColorSpace=}"}},
    )
    r(b"NSObject", b"inputKeys", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"lockBufferRepresentationWithPixelFormat:colorSpace:forBounds:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"^{CGColorSpace=}"},
                4: {
                    "type": sel32or64(
                        b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                },
            },
        },
    )
    r(
        b"NSObject",
        b"lockTextureRepresentationWithColorSpace:forBounds:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {
                2: {"type": b"^{CGColorSpace=}"},
                3: {
                    "type": sel32or64(
                        b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                },
            },
        },
    )
    r(
        b"NSObject",
        b"logMessage:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}},
            "variadic": True,
        },
    )
    r(
        b"NSObject",
        b"outputImageProviderFromBufferWithPixelFormat:pixelsWide:pixelsHigh:baseAddress:bytesPerRow:releaseCallback:releaseContext:colorSpace:shouldColorMatch:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": sel32or64(b"I", b"Q")},
                4: {"type": sel32or64(b"I", b"Q")},
                5: {"type": b"^v"},
                6: {"type": sel32or64(b"I", b"Q")},
                7: {"type": b"^?"},
                8: {"type": b"^v"},
                9: {"type": b"^{CGColorSpace=}"},
                10: {"type": b"Z"},
            },
        },
    )
    r(
        b"NSObject",
        b"outputImageProviderFromTextureWithPixelFormat:pixelsWide:pixelsHigh:name:flipped:releaseCallback:releaseContext:colorSpace:shouldColorMatch:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": sel32or64(b"I", b"Q")},
                4: {"type": sel32or64(b"I", b"Q")},
                5: {"type": b"I"},
                6: {"type": b"Z"},
                7: {"type": b"^?"},
                8: {"type": b"^v"},
                9: {"type": b"^{CGColorSpace=}"},
                10: {"type": b"Z"},
            },
        },
    )
    r(b"NSObject", b"outputKeys", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"propertyListFromInputValues",
        {"required": True, "retval": {"type": b"@"}},
    )
    r(
        b"NSObject",
        b"releaseRenderedTexture:forCGLContext:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"I"}, 3: {"type": b"^{_CGLContextObject=}"}},
        },
    )
    r(
        b"NSObject",
        b"renderToBuffer:withBytesPerRow:pixelFormat:forBounds:",
        {
            "required": False,
            "retval": {"type": b"Z"},
            "arguments": {
                2: {"type": b"^v"},
                3: {"type": sel32or64(b"I", b"Q")},
                4: {"type": b"@"},
                5: {
                    "type": sel32or64(
                        b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                },
            },
        },
    )
    r(
        b"NSObject",
        b"renderWithCGLContext:forBounds:",
        {
            "required": False,
            "retval": {"type": b"Z"},
            "arguments": {
                2: {"type": b"^{_CGLContextObject=}"},
                3: {
                    "type": sel32or64(
                        b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                        b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                },
            },
        },
    )
    r(
        b"NSObject",
        b"setInputValuesWithPropertyList:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"setValue:forInputKey:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(b"NSObject", b"shouldColorMatch", {"required": False, "retval": {"type": b"Z"}})
    r(
        b"NSObject",
        b"supportedBufferPixelFormats",
        {"required": False, "retval": {"type": b"@"}},
    )
    r(
        b"NSObject",
        b"supportedRenderedTexturePixelFormats",
        {"required": False, "retval": {"type": b"@"}},
    )
    r(
        b"NSObject",
        b"textureColorSpace",
        {"required": True, "retval": {"type": b"^{CGColorSpace=}"}},
    )
    r(b"NSObject", b"textureFlipped", {"required": True, "retval": {"type": b"Z"}})
    r(b"NSObject", b"textureMatrix", {"required": True, "retval": {"type": b"^f"}})
    r(b"NSObject", b"textureName", {"required": True, "retval": {"type": b"I"}})
    r(
        b"NSObject",
        b"texturePixelsHigh",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"texturePixelsWide",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(b"NSObject", b"textureTarget", {"required": True, "retval": {"type": b"I"}})
    r(
        b"NSObject",
        b"unbindTextureRepresentationFromCGLContext:textureUnit:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"^{_CGLContextObject=}"}, 3: {"type": b"I"}},
        },
    )
    r(
        b"NSObject",
        b"unlockBufferRepresentation",
        {"required": True, "retval": {"type": b"v"}},
    )
    r(
        b"NSObject",
        b"unlockTextureRepresentation",
        {"required": True, "retval": {"type": b"v"}},
    )
    r(b"NSObject", b"userInfo", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"valueForInputKey:",
        {"required": True, "retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"valueForOutputKey:",
        {"required": True, "retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"valueForOutputKey:ofType:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(b"QCCompositionParameterView", b"drawsBackground", {"retval": {"type": b"Z"}})
    r(b"QCCompositionParameterView", b"hasParameters", {"retval": {"type": b"Z"}})
    r(
        b"QCCompositionParameterView",
        b"setDrawsBackground:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"QCCompositionPickerView", b"allowsEmptySelection", {"retval": {"type": b"Z"}})
    r(b"QCCompositionPickerView", b"drawsBackground", {"retval": {"type": b"Z"}})
    r(b"QCCompositionPickerView", b"isAnimating", {"retval": {"type": b"Z"}})
    r(
        b"QCCompositionPickerView",
        b"setAllowsEmptySelection:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"QCCompositionPickerView",
        b"setDrawsBackground:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"QCCompositionPickerView",
        b"setShowsCompositionNames:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"QCCompositionPickerView", b"showsCompositionNames", {"retval": {"type": b"Z"}})
    r(b"QCPlugIn", b"createViewController", {"retval": {"already_retained": True}})
    r(b"QCPlugIn", b"didValueForInputKeyChange:", {"retval": {"type": b"Z"}})
    r(b"QCPlugIn", b"execute:atTime:withArguments:", {"retval": {"type": b"Z"}})
    r(b"QCPlugIn", b"loadPlugInAtPath:", {"retval": {"type": b"Z"}})
    r(b"QCPlugIn", b"setValue:forOutputKey:", {"retval": {"type": b"Z"}})
    r(b"QCPlugIn", b"startExecution:", {"retval": {"type": b"Z"}})
    r(b"QCRenderer", b"renderAtTime:arguments:", {"retval": {"type": b"Z"}})
    r(b"QCView", b"autostartsRendering", {"retval": {"type": b"Z"}})
    r(b"QCView", b"isPausedRendering", {"retval": {"type": b"Z"}})
    r(b"QCView", b"isRendering", {"retval": {"type": b"Z"}})
    r(b"QCView", b"loadComposition:", {"retval": {"type": b"Z"}})
    r(b"QCView", b"loadCompositionFromFile:", {"retval": {"type": b"Z"}})
    r(b"QCView", b"renderAtTime:arguments:", {"retval": {"type": b"Z"}})
    r(b"QCView", b"setAutostartsRendering:", {"arguments": {2: {"type": b"Z"}}})
    r(b"QCView", b"startRendering", {"retval": {"type": b"Z"}})
    r(
        b"null",
        b"compositionParameterView:shouldDisplayParameterWithKey:attributes:",
        {"retval": {"type": b"Z"}},
    )
    r(b"null", b"didValueForInputKeyChange:", {"retval": {"type": b"Z"}})
    r(b"null", b"loadPlugInAtPath:", {"retval": {"type": b"Z"}})
    r(b"null", b"setValue:forOutputKey:", {"retval": {"type": b"Z"}})
finally:
    objc._updatingMetadata(False)
protocols = {
    "QCCompositionPickerViewDelegate": objc.informal_protocol(
        "QCCompositionPickerViewDelegate",
        [
            objc.selector(
                None,
                b"compositionPickerView:didSelectComposition:",
                b"v@:@@",
                isRequired=False,
            ),
            objc.selector(
                None,
                b"compositionPickerViewWillStopAnimating:",
                b"v@:@",
                isRequired=False,
            ),
            objc.selector(
                None,
                b"compositionPickerViewDidStartAnimating:",
                b"v@:@",
                isRequired=False,
            ),
        ],
    ),
    "QCCompositionParameterViewDelegate": objc.informal_protocol(
        "QCCompositionParameterViewDelegate",
        [
            objc.selector(
                None,
                b"compositionParameterView:didChangeParameterWithKey:",
                b"v@:@@",
                isRequired=False,
            ),
            objc.selector(
                None,
                b"compositionParameterView:shouldDisplayParameterWithKey:attributes:",
                b"Z@:@@@",
                isRequired=False,
            ),
        ],
    ),
}
expressions = {}

# END OF FILE



from math import sin

import objc
import Quartz
import Cocoa
from objc import super
from SampleCIView import SampleCIView

NUM_POINTS = 4


class CIBevelView(SampleCIView):
    currentPoint = objc.ivar(type=objc._C_INT)
    points = objc.ivar()
    angleTime = objc.ivar(type=objc._C_FLT)
    lineImage = objc.ivar()
    twirlFilter = objc.ivar()
    heightFieldFilter = objc.ivar()
    shadedFilter = objc.ivar()

    def initWithFrame_(self, frameRect):
        self = super(CIBevelView, self).initWithFrame_(frameRect)
        if self is None:
            return None

        self.points = [None] * NUM_POINTS
        self.points[0] = Quartz.CGPointMake(
            0.5 * frameRect.size.width, frameRect.size.height - 100.0
        )
        self.points[1] = Quartz.CGPointMake(150.0, 100.0)
        self.points[2] = Quartz.CGPointMake(frameRect.size.width - 150.0, 100.0)
        self.points[3] = Quartz.CGPointMake(
            0.7 * self.points[0].x + 0.3 * self.points[2].x,
            0.7 * self.points[0].y + 0.3 * self.points[2].y,
        )

        url = Cocoa.NSURL.fileURLWithPath_(
            Cocoa.NSBundle.mainBundle().pathForResource_ofType_("lightball", "tiff")
        )

        self.lightball = Quartz.CIImage.imageWithContentsOfURL_(url)

        self.heightFieldFilter = Quartz.CIFilter.filterWithName_(
            "CIHeightFieldFromMask"
        )
        self.heightFieldFilter.setDefaults()
        self.heightFieldFilter.setValue_forKey_(15.0, "inputRadius")

        self.twirlFilter = Quartz.CIFilter.filterWithName_("CITwirlDistortion")
        self.twirlFilter.setDefaults()
        self.twirlFilter.setValue_forKey_(
            Quartz.CIVector.vectorWithX_Y_(
                0.5 * frameRect.size.width, 0.5 * frameRect.size.height
            ),
            "inputCenter",
        )
        self.twirlFilter.setValue_forKey_(300.0, "inputRadius")
        self.twirlFilter.setValue_forKey_(0.0, "inputAngle")

        self.shadedFilter = Quartz.CIFilter.filterWithName_("CIShadedMaterial")
        self.shadedFilter.setDefaults()
        self.shadedFilter.setValue_forKey_(self.lightball, "inputShadingImage")
        self.shadedFilter.setValue_forKey_(20.0, "inputScale")

        # 1/30 second should give us decent animation
        Cocoa.NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(
            1.0 / 30.0, self, "changeTwirlAngle:", None, True
        )
        return self

    def changeTwirlAngle_(self, timer):
        self.angleTime += timer.timeInterval()
        self.twirlFilter.setValue_forKey_(
            -0.2 * sin(self.angleTime * 5.0), "inputAngle"
        )
        self.updateImage()

    def mouseDragged_(self, event):
        loc = self.convertPoint_fromView_(event.locationInWindow(), None)
        self.points[self.currentPoint].x = loc.x
        self.points[self.currentPoint].y = loc.y
        self.lineImage = None

        # normally we'd want this, but the timer will cause us to
        # redisplay anyway
        # self.setNeedsDisplay_(True)

    def mouseDown_(self, event):
        d = 1e4
        loc = self.convertPoint_fromView_(event.locationInWindow(), None)
        for i in range(NUM_POINTS):
            x = self.points[i].x - loc.x
            y = self.points[i].y - loc.y
            t = x * x + y * y

            if t < d:
                self.currentPoint = i
                d = t

        self.mouseDragged_(event)

    def updateImage(self):
        context = Cocoa.NSGraphicsContext.currentContext().CIContext()
        if self.lineImage is None:
            bounds = self.bounds()
            layer = context.createCGLayerWithSize_info_(
                Quartz.CGSizeMake(Cocoa.NSWidth(bounds), Cocoa.NSHeight(bounds)), None
            )

            cg = Quartz.CGLayerGetContext(layer)

            Quartz.CGContextSetRGBStrokeColor(cg, 1, 1, 1, 1)
            Quartz.CGContextSetLineCap(cg, Quartz.kCGLineCapRound)

            Quartz.CGContextSetLineWidth(cg, 60.0)
            Quartz.CGContextMoveToPoint(cg, self.points[0].x, self.points[0].y)
            for i in range(1, NUM_POINTS):
                Quartz.CGContextAddLineToPoint(cg, self.points[i].x, self.points[i].y)
            Quartz.CGContextStrokePath(cg)

            self.lineImage = Quartz.CIImage.alloc().initWithCGLayer_(layer)

        self.heightFieldFilter.setValue_forKey_(self.lineImage, "inputImage")
        self.twirlFilter.setValue_forKey_(
            self.heightFieldFilter.valueForKey_("outputImage"), "inputImage"
        )

        self.shadedFilter.setValue_forKey_(
            self.twirlFilter.valueForKey_("outputImage"), "inputImage"
        )

        self.setImage_(self.shadedFilter.valueForKey_("outputImage"))



"""
Python mapping for the CoreGraphics framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import CoreFoundation
import objc
from Quartz.CoreGraphics import _metadata
from Quartz.CoreGraphics._inlines import _inline_list_

sys.modules["Quartz.CoreGraphics"] = mod = objc.ObjCLazyModule(
    "Quartz.CoreGraphics",
    "com.apple.CoreGraphics",
    objc.pathForFramework(
        "/System/Library/Frameworks/ApplicationServices.framework/Frameworks/CoreGraphics.framework"  # noqa: B950
    ),
    _metadata.__dict__,
    _inline_list_,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (CoreFoundation,),
)


del sys.modules["Quartz.CoreGraphics._metadata"]


def _load(mod):
    import Quartz

    Quartz.CoreGraphics = mod

    # XXX: CGFLOAT_MIN is a #define for FLT_MIN or DBL_MIN, which isn't detected properly
    # by the metadata script.
    import sys

    if sys.maxsize > 1 << 32:
        mod.CGFLOAT_MIN = 1.175_494_350_822_287_5e-38
        mod.CGFLOAT_MAX = 3.402_823_466_385_288_6e38
    else:
        mod.CGFLOAT_MIN = 2.225_073_858_507_201_4e-308
        mod.CGFLOAT_MAX = 1.797_693_134_862_315_7e308

    import Quartz.CoreGraphics._callbacks as m

    for nm in dir(m):
        if nm.startswith("_"):
            continue
        setattr(mod, nm, getattr(m, nm))
    import Quartz.CoreGraphics._doubleindirect as m

    for nm in dir(m):
        if nm.startswith("_"):
            continue
        setattr(mod, nm, getattr(m, nm))
    import Quartz.CoreGraphics._sortandmap as m

    for nm in dir(m):
        if nm.startswith("_"):
            continue
        setattr(mod, nm, getattr(m, nm))
    import Quartz.CoreGraphics._coregraphics as m

    for nm in dir(m):
        if nm.startswith("_"):
            continue
        setattr(mod, nm, getattr(m, nm))
    import Quartz.CoreGraphics._contextmanager as m

    for nm in dir(m):
        if nm.startswith("_"):
            continue
        setattr(mod, nm, getattr(m, nm))

    mod.setCGPathElement(mod.CGPathElement)
    del mod.setCGPathElement

    # a #define
    mod.kCGEventFilterMaskPermitAllEvents = (
        mod.kCGEventFilterMaskPermitLocalMouseEvents
        | mod.kCGEventFilterMaskPermitLocalKeyboardEvents
        | mod.kCGEventFilterMaskPermitSystemDefinedEvents
    )

    def CGEventMaskBit(eventType):
        return 1 << eventType

    mod.CGEventMaskBit = CGEventMaskBit

    mod.kCGColorSpaceUserGray = "kCGColorSpaceUserGray"
    mod.kCGColorSpaceUserRGB = "kCGColorSpaceUserRGB"
    mod.kCGColorSpaceUserCMYK = "kCGColorSpaceUserCMYK"

    # Some pseudo-constants
    mod.kCGBaseWindowLevel = mod.CGWindowLevelForKey(mod.kCGBaseWindowLevelKey)
    mod.kCGMinimumWindowLevel = mod.CGWindowLevelForKey(mod.kCGMinimumWindowLevelKey)
    mod.kCGDesktopWindowLevel = mod.CGWindowLevelForKey(mod.kCGDesktopWindowLevelKey)
    mod.kCGDesktopIconWindowLevel = mod.CGWindowLevelForKey(
        mod.kCGDesktopIconWindowLevelKey
    )
    mod.kCGBackstopMenuLevel = mod.CGWindowLevelForKey(mod.kCGBackstopMenuLevelKey)
    mod.kCGNormalWindowLevel = mod.CGWindowLevelForKey(mod.kCGNormalWindowLevelKey)
    mod.kCGFloatingWindowLevel = mod.CGWindowLevelForKey(mod.kCGFloatingWindowLevelKey)
    mod.kCGTornOffMenuWindowLevel = mod.CGWindowLevelForKey(
        mod.kCGTornOffMenuWindowLevelKey
    )
    mod.kCGDockWindowLevel = mod.CGWindowLevelForKey(mod.kCGDockWindowLevelKey)
    mod.kCGMainMenuWindowLevel = mod.CGWindowLevelForKey(mod.kCGMainMenuWindowLevelKey)
    mod.kCGStatusWindowLevel = mod.CGWindowLevelForKey(mod.kCGStatusWindowLevelKey)
    mod.kCGModalPanelWindowLevel = mod.CGWindowLevelForKey(
        mod.kCGModalPanelWindowLevelKey
    )
    mod.kCGPopUpMenuWindowLevel = mod.CGWindowLevelForKey(
        mod.kCGPopUpMenuWindowLevelKey
    )
    mod.kCGDraggingWindowLevel = mod.CGWindowLevelForKey(mod.kCGDraggingWindowLevelKey)
    mod.kCGScreenSaverWindowLevel = mod.CGWindowLevelForKey(
        mod.kCGScreenSaverWindowLevelKey
    )
    mod.kCGCursorWindowLevel = mod.CGWindowLevelForKey(mod.kCGCursorWindowLevelKey)
    mod.kCGOverlayWindowLevel = mod.CGWindowLevelForKey(mod.kCGOverlayWindowLevelKey)
    mod.kCGHelpWindowLevel = mod.CGWindowLevelForKey(mod.kCGHelpWindowLevelKey)
    mod.kCGUtilityWindowLevel = mod.CGWindowLevelForKey(mod.kCGUtilityWindowLevelKey)
    mod.kCGAssistiveTechHighWindowLevel = mod.CGWindowLevelForKey(
        mod.kCGAssistiveTechHighWindowLevelKey
    )
    mod.kCGMaximumWindowLevel = mod.CGWindowLevelForKey(mod.kCGMaximumWindowLevelKey)

    mod.CGSetLocalEventsFilterDuringSupressionState = (
        mod.CGSetLocalEventsFilterDuringSuppressionState
    )

    mod.kCGAnyInputEventType = 0xFFFFFFFF


_load(mod)



import Quartz
import Utilities


def RedBlackRedRampEvaluate(info, input_value, output_value):
    # The domain of this function is 0 - 1. For an input value of 0
    # this function returns the color to paint at the start point
    # of the shading. For an input value of 1 this function returns
    # the color to paint at the end point of the shading. This
    # is a 1 in, 4 out function where the output values correspond
    # to an r,g,b,a color.
    #
    # For an RGB color space as the shading color space, this
    # function evaluates to produce a blend from pure, opaque
    # red at the start point to a pure opaque black at the
    # midpoint, and back to pure opaque red at the end point.

    return (
        # The red component evaluates to 1 for an input value of 0
        # (the start point of the shading). It smoothly reduces
        # to zero at the midpoint of the shading (input value 0.5)
        # and increases up to 1 at the endpoint of the shading (input
        # value 1.0).
        abs(1.0 - input_value[0] * 2),
        # The green and blue components are always 0.
        0,
        0,
        # The alpha component is 1 for the entire shading.
        1,
    )


def createFunctionForRGB(evaluationFunction):
    # This is a 1 in, 4 out function for drawing shadings
    # in a 3 component (plus alpha) color space. Shadings
    # parameterize the endpoints such that the starting point
    # represents the function input value 0 and the ending point
    # represents the function input value 1.
    domain = (0, 1)

    # The range is the range for the output colors. For an rgb
    # color space the values range from 0-1 for the r,g,b, and a
    # components.

    function_range = (
        # The red component, min and max.
        0,
        1,
        # The green component, min and max.
        0,
        1,
        # The blue component, min and max.
        0,
        1,
        # The alpha component, min and max.
        0,
        1,
    )

    # Dimension of domain is 1 and dimension of range is 4.
    function = Quartz.CGFunctionCreate(
        None, 1, domain, 4, function_range, evaluationFunction
    )

    if function is None:
        print("Couldn't create the CGFunction!")
        return None

    return function


def doSimpleAxialShading(context):
    # This shading paints colors in the calibrated Generic RGB
    # color space so it needs a function that evaluates 1 in to 4 out.
    axialFunction = createFunctionForRGB(RedBlackRedRampEvaluate)
    if axialFunction is None:
        return

    # Start the shading at the point (20,20) and
    # end it at (420,20). The axis of the shading
    # is a line from (20,20) to (420,20).
    startPoint = Quartz.CGPoint(x=20, y=20)
    endPoint = Quartz.CGPoint(x=420, y=20)

    # Don't extend this shading.
    extendStart = extendEnd = False

    shading = Quartz.CGShadingCreateAxial(
        Utilities.getTheCalibratedRGBColorSpace(),
        startPoint,
        endPoint,
        axialFunction,
        extendStart,
        extendEnd,
    )
    # The shading retains the function and this code
    # is done with the function so it should release it.
    del axialFunction
    if shading is None:
        print("Couldn't create the shading!")
        return

    # Draw the shading. This paints the shading to
    # the destination context, clipped by the
    # current clipping area.
    Quartz.CGContextDrawShading(context, shading)


def RedGreenRampEvaluate(info, input_value, output_value):
    # The domain of this function is 0 - 1. For an input value of 0
    # this function returns the color to paint at the start point
    # of the shading. For an input value of 1 this function returns
    # the color to paint at the end point of the shading. This
    # is a 1 in, 4 out function where the output values correspond
    # to an r,g,b,a color.
    #
    # For an RGB color space as the shading color space, this
    # function evaluates to produce a blend from pure, opaque
    # red at the start point to a pure opaque green at the end point.

    return (
        # The red component starts at 1 and reduces to zero as the input
        # goes from 0 (the start point of the shading) and increases
        # to 1 (the end point of the shading).
        1.0 - input_value[0],
        # The green component starts at 0 for an input of 0
        # (the start point of the shading) and increases to 1
        # for an input value of 1 (the end point of the shading).
        input_value[0],
        # The blue component is always 0.
        0,
        # The alpha component is always 1, the shading is always opaque.
        1,
    )


def doExampleAxialShading(context):
    rect = Quartz.CGRectMake(0, 0, 240, 240)

    # This shading paints colors in the calibrated Generic RGB
    # color space so it needs a function that evaluates 1 in to 4 out.
    redGreenFunction = createFunctionForRGB(RedGreenRampEvaluate)
    if redGreenFunction is None:
        return

    # Start the shading at the point (20,20) and
    # end it at (220,220). The axis of the shading
    # is a diagonal line from (20,20) to (220,220).
    startPoint = Quartz.CGPoint(x=20, y=20)
    endPoint = Quartz.CGPoint(x=220, y=220)

    # Don't extend this shading.
    extendStart = extendEnd = False
    shading = Quartz.CGShadingCreateAxial(
        Utilities.getTheCalibratedRGBColorSpace(),
        startPoint,
        endPoint,
        redGreenFunction,
        extendStart,
        extendEnd,
    )

    if shading is None:
        print("Couldn't create the shading!")
        return

    # Position for the first portion of the drawing.
    Quartz.CGContextTranslateCTM(context, 40, 260)

    # Stroke a black rectangle that will frame the shading.
    Quartz.CGContextSetLineWidth(context, 2)
    Quartz.CGContextSetStrokeColorWithColor(context, Utilities.getRGBOpaqueBlackColor())
    Quartz.CGContextStrokeRect(context, rect)

    Quartz.CGContextSaveGState(context)
    if 1:
        # Clip to the rectangle that was just stroked.
        Quartz.CGContextClipToRect(context, rect)
        # Draw the shading. This paints the shading to
        # the destination context, clipped to rect.
        Quartz.CGContextDrawShading(context, shading)
        # Release the shading once the code is finished with it.
        del shading
        # Restore the graphics state so that the rectangular
        # clip is no longer present.
    Quartz.CGContextRestoreGState(context)

    # Prepare for the next shading.
    Quartz.CGContextTranslateCTM(context, 0, -250)

    # Extend this shading.
    extendStart = extendEnd = True
    shading = Quartz.CGShadingCreateAxial(
        Utilities.getTheCalibratedRGBColorSpace(),
        startPoint,
        endPoint,
        redGreenFunction,
        extendStart,
        extendEnd,
    )
    # The shading retains the function and this code
    # is done with the function so it should release it.
    del redGreenFunction
    if shading is None:
        print("Couldn't create the shading!")
        return

    # Stroke with the current stroke color.
    Quartz.CGContextStrokeRect(context, rect)

    Quartz.CGContextSaveGState(context)
    if 1:
        Quartz.CGContextClipToRect(context, rect)
        # Draw the shading. This paints the shading to
        # the destination context, clipped to rect.
        Quartz.CGContextDrawShading(context, shading)
    Quartz.CGContextRestoreGState(context)

    # Now paint some text with a shading.
    Quartz.CGContextSaveGState(context)
    if 1:
        Quartz.CGContextTranslateCTM(context, 260, 0)
        Quartz.CGContextSetTextMatrix(context, Quartz.CGAffineTransformIdentity)

        # Set the font with the PostScript name "Times-Roman", at
        # 80 points, with the MacRoman encoding.
        Quartz.CGContextSelectFont(
            context, b"Times-Roman", 80, Quartz.kCGEncodingMacRoman
        )

        # Rotate so that the text characters are rotated
        # relative to the page.
        Quartz.CGContextRotateCTM(context, Utilities.DEGREES_TO_RADIANS(45))
        # Set the text drawing mode to clip so that
        # the characters in the string are intersected with
        # the clipping area.
        Quartz.CGContextSetTextDrawingMode(context, Quartz.kCGTextClip)
        Quartz.CGContextShowTextAtPoint(context, 30, 0, b"Shading", 7)

        # At this point nothing has been painted; the
        # glyphs in the word "Shading" have been intersected
        # with the previous clipping area to create a new
        # clipping area.

        # Rotate the coordinate system back so that the
        # shading is not rotated relative to the page.
        Quartz.CGContextRotateCTM(context, Utilities.DEGREES_TO_RADIANS(-45))

        # Draw the shading, painting the shading
        # to the destination context, clipped by the glyphs.
        Quartz.CGContextDrawShading(context, shading)

    Quartz.CGContextRestoreGState(context)
    # Release the shading once the code is finished with it.
    del shading


class MyStartEndColor(object):
    def __init__(self):
        self.startColor = [0.0] * 3
        self.endColor = [0.0] * 3


def StartColorEndColorEvaluate(info, input_value, output_value):
    # The domain of this function is 0 - 1. For an input value of 0
    # this function returns the color to paint at the start point
    # of the shading. For an input value of 1 this function returns
    # the color to paint at the end point of the shading. This
    # is a 1 in, 4 out function where the output values correspond
    # to an r,g,b,a color.
    #
    # This function evaluates to produce a blend from startColor to endColor.
    #
    # Note that the returned results are clipped to the range
    # by Quartz so this function doesn't worry about values
    # that are outside the range 0-1.

    # Weight the starting and ending color components depending
    # on what position in the blend the input value specifies.
    return (
        (info.startColor[0] * (1 - input_value[0]) + info.endColor[0] * input_value[0]),
        (info.startColor[1] * (1 - input_value[0]) + info.endColor[1] * input_value[0]),
        (info.startColor[2] * (1 - input_value[0]) + info.endColor[2] * input_value[0]),
        # The alpha component is always 1, the shading is always opaque.
        1,
    )


def createFunctionWithStartEndColorRamp(startColor, endColor):
    # Use a pointer to a MyStartEndColor as a way of
    # parameterizing the color ramp this function produces.
    startEndColorP = MyStartEndColor()

    # Set up start and end colors in the info structure.
    startEndColorP.startColor[0] = startColor[0]
    startEndColorP.startColor[1] = startColor[1]
    startEndColorP.startColor[2] = startColor[2]

    startEndColorP.endColor[0] = endColor[0]
    startEndColorP.endColor[1] = endColor[1]
    startEndColorP.endColor[2] = endColor[2]

    # This is a 1 in, 4 out function for drawing shadings
    # in a 3 component (plus alpha) color space. Shadings
    # parameterize the endpoints such that the starting point
    # represents the function input value 0 and the ending point
    # represents the function input value 1.
    domain = (0, 1)

    # The range is the range for the output colors. For an rgb
    # color space the values range from 0-1 for the r,g,b, and a
    # components.

    function_range = (
        # The red component, min and max.
        0,
        1,
        # The green component, min and max.
        0,
        1,
        # The blue component, min and max.
        0,
        1,
        # The alpha component, min and max.
        0,
        1,
    )

    # Pass startEndColorP as the info parameter.
    function = Quartz.CGFunctionCreate(
        startEndColorP, 1, domain, 4, function_range, StartColorEndColorEvaluate
    )

    if function is None:
        print("Couldn't create the CGFunction!")
        return None

    return function


def doSimpleRadialShading(context):
    startColor = [0.663, 0.0, 0.031]  # Red.
    endColor = [1.0, 0.8, 0.4]  # Light yellow.

    # This function describes a color ramp where the starting color
    # is red and the ending color is blue.
    redYellowFunction = createFunctionWithStartEndColorRamp(startColor, endColor)
    if redYellowFunction is None:
        return

    Quartz.CGContextTranslateCTM(context, 120, 120)

    # Circles whose origin is the same.
    circleACenter = Quartz.CGPoint(x=0, y=0)
    circleBCenter = circleACenter

    # The starting circle is inside the ending circle.
    circleARadius = 50
    circleBRadius = 100

    #  Don't extend the shading.
    extendStart = extendEnd = False
    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        redYellowFunction,
        extendStart,
        extendEnd,
    )

    del redYellowFunction
    if shading is None:
        print("Couldn't create the shading!")
        return
    Quartz.CGContextDrawShading(context, shading)


def doExampleRadialShadings(context):
    magenta = [1, 0, 1]  # Pure magenta.
    magenta30 = [0.3, 0, 0.3]  # 30% magenta.
    black = [0, 0, 0]
    red = [1, 0, 0]
    green = [0, 1, 0]
    blue = [0, 0, 1]
    redgreen = [0.66, 1, 0.04]  # A red-green shade.

    Quartz.CGContextTranslateCTM(context, 120, 550)

    # This function describes a color ramp where the starting color
    # is a full magenta, the ending color is 30% magenta.
    magentaFunction = createFunctionWithStartEndColorRamp(magenta, magenta30)
    if magentaFunction is None:
        print("Couldn't create the magenta function!")
        return

    # Shading 1. Circle A is completely inside circle B but with
    # different origins. Circle A has radius 0 which produces
    # a point source.

    # The center of circle A is offset from the origin.
    circleACenter = Quartz.CGPoint(x=30, y=40)
    # The center of circle B is at the origin.
    circleBCenter = Quartz.CGPoint(x=0, y=0)

    # A radius of zero produces a point source.
    circleARadius = 0
    circleBRadius = 100

    # Don't extend the shading.
    extendStart = extendEnd = False
    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        magentaFunction,
        extendStart,
        extendEnd,
    )
    # Finished with the magenta function so release it.
    del magentaFunction
    if shading is None:
        print("Couldn't create the shading!")
        return

    Quartz.CGContextDrawShading(context, shading)
    # Finished with the shading so release it.
    del shading

    # Shading 2. Circle A is completely inside
    # circle B but with different origins.

    # The starting color is red and the ending color is green.
    redGreenFunction = createFunctionWithStartEndColorRamp(red, green)
    if redGreenFunction is None:
        print("Couldn't create the red-Green function!")
        return

    circleACenter.x = 55
    circleACenter.y = 70
    circleBCenter.x = 20
    circleBCenter.y = 0
    circleARadius = 10
    # The outer circle is outside the clipping path so the
    # color at the edge of the shape is not
    # that at the radius of the outer circle.
    circleBRadius = 200
    # Extend the end point of this shading.
    extendStart = False
    extendEnd = True
    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        redGreenFunction,
        extendStart,
        extendEnd,
    )
    # Finished with this function so release it.
    del redGreenFunction
    if shading is None:
        print("Couldn't create the shading!")
        return

    # Set a clipping area to bound the extend. This code
    # sets a clipping area that corresponds to a circular
    # wedge. The starting circle is inside the clipping
    # area and the ending circle is outside.
    Quartz.CGContextSaveGState(context)
    if 1:
        Quartz.CGContextTranslateCTM(context, 250, 0)
        Quartz.CGContextBeginPath(context)
        Quartz.CGContextMoveToPoint(context, 25, 0)
        Quartz.CGContextAddArc(
            context,
            25,
            0,
            130,
            Utilities.DEGREES_TO_RADIANS(30),
            Utilities.DEGREES_TO_RADIANS(-30),
            0,
        )
        Quartz.CGContextClip(context)
        # Paint the shading.
        Quartz.CGContextDrawShading(context, shading)
        # Finished with the shading so release it.
        del shading
    Quartz.CGContextRestoreGState(context)

    Quartz.CGContextTranslateCTM(context, -40, -250)

    # Shading 3. The starting circle is completely outside
    # the ending circle, no extension. The circles
    # have the same radii.
    circleACenter.x = 0
    circleACenter.y = 0
    circleBCenter.x = 125
    circleBCenter.y = 0

    circleARadius = 50
    circleBRadius = 50

    extendStart = extendEnd = False

    # Create a function that paints a red to black ramp.
    redBlackFunction = createFunctionWithStartEndColorRamp(red, black)
    if redBlackFunction is None:
        print("Couldn't create the red-black function!")
        return

    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        redBlackFunction,
        extendStart,
        extendEnd,
    )
    if shading is None:
        print("Couldn't create the shading!")
        return

    Quartz.CGContextDrawShading(context, shading)
    # Finished with the shading so release it.
    del shading

    # Shading 4. The starting circle is completely outside
    # the ending circle. The circles have different radii.
    circleACenter.x = 120
    circleACenter.y = 0
    circleBCenter.x = 0
    circleBCenter.y = 0

    circleARadius = 75
    circleBRadius = 30

    # Extend at the start and end.
    extendStart = extendEnd = True
    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        redBlackFunction,
        extendStart,
        extendEnd,
    )
    # Finished with this function so release it.
    del redBlackFunction
    if shading is None:
        print("Couldn't create the shading!")
        return

    Quartz.CGContextSaveGState(context)
    if 1:
        Quartz.CGContextTranslateCTM(context, 270, 0)
        # Clip to an elliptical path so the shading
        # does not extend to infinity at the larger end.
        Quartz.CGContextBeginPath(context)
        Utilities.myCGContextAddEllipseInRect(
            context, Quartz.CGRectMake(-200, -200, 450, 400)
        )
        Quartz.CGContextClip(context)
        Quartz.CGContextDrawShading(context, shading)
        # Finished with the shading so release it.
        del shading
    Quartz.CGContextRestoreGState(context)

    Quartz.CGContextTranslateCTM(context, 30, -200)

    # The starting color is blue, the ending color is a red-green color.
    blueGreenFunction = createFunctionWithStartEndColorRamp(blue, redgreen)
    if blueGreenFunction is None:
        print("Couldn't create the blue-Green function!")
        return

    # Shading 5. The circles partially overlap and have
    # different radii with the larger circle at the start.
    circleACenter.x = 0
    circleACenter.y = 0
    circleBCenter.x = 90
    circleBCenter.y = 30

    circleARadius = 75
    circleBRadius = 45

    extendStart = extendEnd = False
    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        blueGreenFunction,
        extendStart,
        extendEnd,
    )
    if shading is None:
        print("Couldn't create the shading!")
        return

    Quartz.CGContextDrawShading(context, shading)
    # Finished with the shading so release it.
    del shading

    Quartz.CGContextTranslateCTM(context, 200, 0)

    # Shading 6. The circles partially overlap and have
    # different radii with the larger circle at the end.

    circleARadius = 45
    circleBRadius = 75
    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        blueGreenFunction,
        extendStart,
        extendEnd,
    )
    # Finished with this function so release it.
    del blueGreenFunction
    if shading is None:
        print("Couldn't create the shading!")
        return

    Quartz.CGContextDrawShading(context, shading)


def doEllipseShading(context):
    black = [0, 0, 0]
    red = [1, 0, 0]

    # This function describes a color ramp where the starting color
    # is red, the ending color is black.
    redBlackFunction = createFunctionWithStartEndColorRamp(red, black)
    if redBlackFunction is None:
        print("Couldn't create the red-black function!")
        return

    Quartz.CGContextTranslateCTM(context, 100, 300)
    # Shading 1.
    #   To obtain an elliptical shading requires that user space
    #   at the time the shading is painted is transformed so that
    #   the circles which define the radial shading geometry are
    #   rotated and elliptical. User space will be rotated
    #   by 45 degrees, then scaled by 1 in x and 2 in y to produce
    #   the ellipses.

    # Compute the transform needed to create the rotated ellipses.
    t = Quartz.CGAffineTransformMakeRotation(Utilities.DEGREES_TO_RADIANS(45))
    t = Quartz.CGAffineTransformScale(t, 1, 2)

    circleACenter = Quartz.CGPoint(x=0, y=0)
    circleBCenter = Quartz.CGPoint(x=circleACenter.x + 144, y=circleACenter.y)
    circleARadius = 45
    circleBRadius = 45

    # Don't extend this shading.
    extendStart = extendEnd = False

    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        redBlackFunction,
        extendStart,
        extendEnd,
    )
    if shading is None:
        # Couldn't create the shading so release
        # the function before returning.
        print("Couldn't create the shading!")
        return

    Quartz.CGContextSaveGState(context)
    if 1:
        # Transform coordinates for the drawing of the shading.
        # This transform produces the rotated elliptical shading.
        # This produces the left shading in the figure, the
        # one where both the ellipses and the shading are
        # rotated relative to default user space.
        Quartz.CGContextConcatCTM(context, t)

        Quartz.CGContextDrawShading(context, shading)
        del shading
    Quartz.CGContextRestoreGState(context)

    Quartz.CGContextTranslateCTM(context, 300, 10)

    # Shading 2.
    #    Now draw the shading where the shading ellipses are
    #    rotated but the axis between the origins of
    #    the ellipses lies parallel to the x axis in default
    #    user space. This is similar to the shading drawn
    #    manually in Chapter 5.
    #
    #    To compute the correct origins for the shading,
    #    the code needs to compute the points that,
    #    transformed by the matrix t used to paint the shading,
    #    produce the desired coordinates. We want coordinates
    #    that are transformed as follows:
    #
    #            P' = P x t
    #
    #    where P' is the point in untransformed user space that
    #    we want as the origin, P is the point in transformed
    #    user space that will be transformed by t, the matrix
    #    which transforms the circles into rotated ellipses.
    #
    #    So we want to calculate P such that P' = P x t .
    #
    #    Notice that if P = P' x Inverse(t) then:
    #
    #    P' = P' x Inverse(t) x t = P' x Identity = P'.
    #
    #    This means that we can calculate the point P
    #    by computing P' x Inverse(t).

    inverseT = Quartz.CGAffineTransformInvert(t)
    # Now the code can transform the coordinates through the
    # inverse transform to compute the new coordinates. These
    # coordinates, when transformed with the transform t,
    # produce the original coordinate.
    circleACenter = Quartz.CGPointApplyAffineTransform(circleACenter, inverseT)
    circleBCenter = Quartz.CGPointApplyAffineTransform(circleBCenter, inverseT)

    shading = Quartz.CGShadingCreateRadial(
        Utilities.getTheCalibratedRGBColorSpace(),
        circleACenter,
        circleARadius,
        circleBCenter,
        circleBRadius,
        redBlackFunction,
        extendStart,
        extendEnd,
    )
    # The code is finished with the function so release it.
    del redBlackFunction
    if shading is None:
        print("Couldn't create the shading!")
        return

    # Transform coordinates for the drawing of the shading.
    # This transform produces the rotated elliptical shading.
    Quartz.CGContextConcatCTM(context, t)

    Quartz.CGContextDrawShading(context, shading)
